# This file is part of cloud-init. See LICENSE file for license information.

from cloudinit.config.cc_ubuntu_advantage import (
    configure_ua, handle, maybe_install_ua_tools, schema)
from cloudinit.config.schema import validate_cloudconfig_schema
from cloudinit import subp
from cloudinit.tests.helpers import (
    CiTestCase, mock, SchemaTestCaseMixin, skipUnlessJsonSchema)


# Module path used in mocks
MPATH = 'cloudinit.config.cc_ubuntu_advantage'


class FakeCloud(object):
    def __init__(self, distro):
        self.distro = distro


class TestConfigureUA(CiTestCase):

    with_logs = True
    allowed_subp = [CiTestCase.SUBP_SHELL_TRUE]

    def setUp(self):
        super(TestConfigureUA, self).setUp()
        self.tmp = self.tmp_dir()

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_error(self, m_subp):
        """Errors from ua attach command are raised."""
        m_subp.side_effect = subp.ProcessExecutionError(
            'Invalid token SomeToken')
        with self.assertRaises(RuntimeError) as context_manager:
            configure_ua(token='SomeToken')
        self.assertEqual(
            'Failure attaching Ubuntu Advantage:\nUnexpected error while'
            ' running command.\nCommand: -\nExit code: -\nReason: -\n'
            'Stdout: Invalid token SomeToken\nStderr: -',
            str(context_manager.exception))

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_with_token(self, m_subp):
        """When token is provided, attach the machine to ua using the token."""
        configure_ua(token='SomeToken')
        m_subp.assert_called_once_with(['ua', 'attach', 'SomeToken'])
        self.assertEqual(
            'DEBUG: Attaching to Ubuntu Advantage. ua attach SomeToken\n',
            self.logs.getvalue())

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_on_service_error(self, m_subp):
        """all services should be enabled and then any failures raised"""

        def fake_subp(cmd, capture=None):
            fail_cmds = [['ua', 'enable', svc] for svc in ['esm', 'cc']]
            if cmd in fail_cmds and capture:
                svc = cmd[-1]
                raise subp.ProcessExecutionError(
                    'Invalid {} credentials'.format(svc.upper()))

        m_subp.side_effect = fake_subp

        with self.assertRaises(RuntimeError) as context_manager:
            configure_ua(token='SomeToken', enable=['esm', 'cc', 'fips'])
        self.assertEqual(
            m_subp.call_args_list,
            [mock.call(['ua', 'attach', 'SomeToken']),
             mock.call(['ua', 'enable', 'esm'], capture=True),
             mock.call(['ua', 'enable', 'cc'], capture=True),
             mock.call(['ua', 'enable', 'fips'], capture=True)])
        self.assertIn(
            'WARNING: Failure enabling "esm":\nUnexpected error'
            ' while running command.\nCommand: -\nExit code: -\nReason: -\n'
            'Stdout: Invalid ESM credentials\nStderr: -\n',
            self.logs.getvalue())
        self.assertIn(
            'WARNING: Failure enabling "cc":\nUnexpected error'
            ' while running command.\nCommand: -\nExit code: -\nReason: -\n'
            'Stdout: Invalid CC credentials\nStderr: -\n',
            self.logs.getvalue())
        self.assertEqual(
            'Failure enabling Ubuntu Advantage service(s): "esm", "cc"',
            str(context_manager.exception))

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_with_empty_services(self, m_subp):
        """When services is an empty list, do not auto-enable attach."""
        configure_ua(token='SomeToken', enable=[])
        m_subp.assert_called_once_with(['ua', 'attach', 'SomeToken'])
        self.assertEqual(
            'DEBUG: Attaching to Ubuntu Advantage. ua attach SomeToken\n',
            self.logs.getvalue())

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_with_specific_services(self, m_subp):
        """When services a list, only enable specific services."""
        configure_ua(token='SomeToken', enable=['fips'])
        self.assertEqual(
            m_subp.call_args_list,
            [mock.call(['ua', 'attach', 'SomeToken']),
             mock.call(['ua', 'enable', 'fips'], capture=True)])
        self.assertEqual(
            'DEBUG: Attaching to Ubuntu Advantage. ua attach SomeToken\n',
            self.logs.getvalue())

    @mock.patch('%s.maybe_install_ua_tools' % MPATH, mock.MagicMock())
    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_with_string_services(self, m_subp):
        """When services a string, treat as singleton list and warn"""
        configure_ua(token='SomeToken', enable='fips')
        self.assertEqual(
            m_subp.call_args_list,
            [mock.call(['ua', 'attach', 'SomeToken']),
             mock.call(['ua', 'enable', 'fips'], capture=True)])
        self.assertEqual(
            'WARNING: ubuntu_advantage: enable should be a list, not a'
            ' string; treating as a single enable\n'
            'DEBUG: Attaching to Ubuntu Advantage. ua attach SomeToken\n',
            self.logs.getvalue())

    @mock.patch('%s.subp.subp' % MPATH)
    def test_configure_ua_attach_with_weird_services(self, m_subp):
        """When services not string or list, warn but still attach"""
        configure_ua(token='SomeToken', enable={'deffo': 'wont work'})
        self.assertEqual(
            m_subp.call_args_list,
            [mock.call(['ua', 'attach', 'SomeToken'])])
        self.assertEqual(
            'WARNING: ubuntu_advantage: enable should be a list, not a'
            ' dict; skipping enabling services\n'
            'DEBUG: Attaching to Ubuntu Advantage. ua attach SomeToken\n',
            self.logs.getvalue())


@skipUnlessJsonSchema()
class TestSchema(CiTestCase, SchemaTestCaseMixin):

    with_logs = True
    schema = schema

    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    @mock.patch('%s.configure_ua' % MPATH)
    def test_schema_warns_on_ubuntu_advantage_not_dict(self, _cfg, _):
        """If ubuntu_advantage configuration is not a dict, emit a warning."""
        validate_cloudconfig_schema({'ubuntu_advantage': 'wrong type'}, schema)
        self.assertEqual(
            "WARNING: Invalid config:\nubuntu_advantage: 'wrong type' is not"
            " of type 'object'\n",
            self.logs.getvalue())

    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    @mock.patch('%s.configure_ua' % MPATH)
    def test_schema_disallows_unknown_keys(self, _cfg, _):
        """Unknown keys in ubuntu_advantage configuration emit warnings."""
        validate_cloudconfig_schema(
            {'ubuntu_advantage': {'token': 'winner', 'invalid-key': ''}},
            schema)
        self.assertIn(
            'WARNING: Invalid config:\nubuntu_advantage: Additional properties'
            " are not allowed ('invalid-key' was unexpected)",
            self.logs.getvalue())

    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    @mock.patch('%s.configure_ua' % MPATH)
    def test_warn_schema_requires_token(self, _cfg, _):
        """Warn if ubuntu_advantage configuration lacks token."""
        validate_cloudconfig_schema(
            {'ubuntu_advantage': {'enable': ['esm']}}, schema)
        self.assertEqual(
            "WARNING: Invalid config:\nubuntu_advantage:"
            " 'token' is a required property\n", self.logs.getvalue())

    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    @mock.patch('%s.configure_ua' % MPATH)
    def test_warn_schema_services_is_not_list_or_dict(self, _cfg, _):
        """Warn when ubuntu_advantage:enable config is not a list."""
        validate_cloudconfig_schema(
            {'ubuntu_advantage': {'enable': 'needslist'}}, schema)
        self.assertEqual(
            "WARNING: Invalid config:\nubuntu_advantage: 'token' is a"
            " required property\nubuntu_advantage.enable: 'needslist'"
            " is not of type 'array'\n",
            self.logs.getvalue())


class TestHandle(CiTestCase):

    with_logs = True

    def setUp(self):
        super(TestHandle, self).setUp()
        self.tmp = self.tmp_dir()

    @mock.patch('%s.validate_cloudconfig_schema' % MPATH)
    def test_handle_no_config(self, m_schema):
        """When no ua-related configuration is provided, nothing happens."""
        cfg = {}
        handle('ua-test', cfg=cfg, cloud=None, log=self.logger, args=None)
        self.assertIn(
            "DEBUG: Skipping module named ua-test, no 'ubuntu_advantage'"
            ' configuration found',
            self.logs.getvalue())
        m_schema.assert_not_called()

    @mock.patch('%s.configure_ua' % MPATH)
    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    def test_handle_tries_to_install_ubuntu_advantage_tools(
            self, m_install, m_cfg):
        """If ubuntu_advantage is provided, try installing ua-tools package."""
        cfg = {'ubuntu_advantage': {'token': 'valid'}}
        mycloud = FakeCloud(None)
        handle('nomatter', cfg=cfg, cloud=mycloud, log=self.logger, args=None)
        m_install.assert_called_once_with(mycloud)

    @mock.patch('%s.configure_ua' % MPATH)
    @mock.patch('%s.maybe_install_ua_tools' % MPATH)
    def test_handle_passes_credentials_and_services_to_configure_ua(
            self, m_install, m_configure_ua):
        """All ubuntu_advantage config keys are passed to configure_ua."""
        cfg = {'ubuntu_advantage': {'token': 'token', 'enable': ['esm']}}
        handle('nomatter', cfg=cfg, cloud=None, log=self.logger, args=None)
        m_configure_ua.assert_called_once_with(
            token='token', enable=['esm'])

    @mock.patch('%s.maybe_install_ua_tools' % MPATH, mock.MagicMock())
    @mock.patch('%s.configure_ua' % MPATH)
    def test_handle_warns_on_deprecated_ubuntu_advantage_key_w_config(
            self, m_configure_ua):
        """Warning when ubuntu-advantage key is present with new config"""
        cfg = {'ubuntu-advantage': {'token': 'token', 'enable': ['esm']}}
        handle('nomatter', cfg=cfg, cloud=None, log=self.logger, args=None)
        self.assertEqual(
            'WARNING: Deprecated configuration key "ubuntu-advantage"'
            ' provided. Expected underscore delimited "ubuntu_advantage";'
            ' will attempt to continue.',
            self.logs.getvalue().splitlines()[0])
        m_configure_ua.assert_called_once_with(
            token='token', enable=['esm'])

    def test_handle_error_on_deprecated_commands_key_dashed(self):
        """Error when commands is present in ubuntu-advantage key."""
        cfg = {'ubuntu-advantage': {'commands': 'nogo'}}
        with self.assertRaises(RuntimeError) as context_manager:
            handle('nomatter', cfg=cfg, cloud=None, log=self.logger, args=None)
        self.assertEqual(
            'Deprecated configuration "ubuntu-advantage: commands" provided.'
            ' Expected "token"',
            str(context_manager.exception))

    def test_handle_error_on_deprecated_commands_key_underscored(self):
        """Error when commands is present in ubuntu_advantage key."""
        cfg = {'ubuntu_advantage': {'commands': 'nogo'}}
        with self.assertRaises(RuntimeError) as context_manager:
            handle('nomatter', cfg=cfg, cloud=None, log=self.logger, args=None)
        self.assertEqual(
            'Deprecated configuration "ubuntu-advantage: commands" provided.'
            ' Expected "token"',
            str(context_manager.exception))

    @mock.patch('%s.maybe_install_ua_tools' % MPATH, mock.MagicMock())
    @mock.patch('%s.configure_ua' % MPATH)
    def test_handle_prefers_new_style_config(
            self, m_configure_ua):
        """ubuntu_advantage should be preferred over ubuntu-advantage"""
        cfg = {
            'ubuntu-advantage': {'token': 'nope', 'enable': ['wrong']},
            'ubuntu_advantage': {'token': 'token', 'enable': ['esm']},
        }
        handle('nomatter', cfg=cfg, cloud=None, log=self.logger, args=None)
        self.assertEqual(
            'WARNING: Deprecated configuration key "ubuntu-advantage"'
            ' provided. Expected underscore delimited "ubuntu_advantage";'
            ' will attempt to continue.',
            self.logs.getvalue().splitlines()[0])
        m_configure_ua.assert_called_once_with(
            token='token', enable=['esm'])


class TestMaybeInstallUATools(CiTestCase):

    with_logs = True

    def setUp(self):
        super(TestMaybeInstallUATools, self).setUp()
        self.tmp = self.tmp_dir()

    @mock.patch('%s.subp.which' % MPATH)
    def test_maybe_install_ua_tools_noop_when_ua_tools_present(self, m_which):
        """Do nothing if ubuntu-advantage-tools already exists."""
        m_which.return_value = '/usr/bin/ua'  # already installed
        distro = mock.MagicMock()
        distro.update_package_sources.side_effect = RuntimeError(
            'Some apt error')
        maybe_install_ua_tools(cloud=FakeCloud(distro))  # No RuntimeError

    @mock.patch('%s.subp.which' % MPATH)
    def test_maybe_install_ua_tools_raises_update_errors(self, m_which):
        """maybe_install_ua_tools logs and raises apt update errors."""
        m_which.return_value = None
        distro = mock.MagicMock()
        distro.update_package_sources.side_effect = RuntimeError(
            'Some apt error')
        with self.assertRaises(RuntimeError) as context_manager:
            maybe_install_ua_tools(cloud=FakeCloud(distro))
        self.assertEqual('Some apt error', str(context_manager.exception))
        self.assertIn('Package update failed\nTraceback', self.logs.getvalue())

    @mock.patch('%s.subp.which' % MPATH)
    def test_maybe_install_ua_raises_install_errors(self, m_which):
        """maybe_install_ua_tools logs and raises package install errors."""
        m_which.return_value = None
        distro = mock.MagicMock()
        distro.update_package_sources.return_value = None
        distro.install_packages.side_effect = RuntimeError(
            'Some install error')
        with self.assertRaises(RuntimeError) as context_manager:
            maybe_install_ua_tools(cloud=FakeCloud(distro))
        self.assertEqual('Some install error', str(context_manager.exception))
        self.assertIn(
            'Failed to install ubuntu-advantage-tools\n', self.logs.getvalue())

    @mock.patch('%s.subp.which' % MPATH)
    def test_maybe_install_ua_tools_happy_path(self, m_which):
        """maybe_install_ua_tools installs ubuntu-advantage-tools."""
        m_which.return_value = None
        distro = mock.MagicMock()  # No errors raised
        maybe_install_ua_tools(cloud=FakeCloud(distro))
        distro.update_package_sources.assert_called_once_with()
        distro.install_packages.assert_called_once_with(
            ['ubuntu-advantage-tools'])

# vi: ts=4 expandtab



"""Integration test for the ntp module's ``servers`` functionality with ntp.

This test specifies the use of the `ntp` NTP client, and ensures that the given
NTP servers are configured as expected.

(This is ported from ``tests/cloud_tests/testcases/modules/ntp_servers.yaml``.)
"""
import re

import yaml
import pytest

USER_DATA = """\
#cloud-config
ntp:
  ntp_client: ntp
  servers:
      - 172.16.15.14
      - 172.16.17.18
"""

EXPECTED_SERVERS = yaml.safe_load(USER_DATA)["ntp"]["servers"]


@pytest.mark.ci
@pytest.mark.user_data(USER_DATA)
class TestNtpServers:

    def test_ntp_installed(self, class_client):
        """Test that `ntpd --version` succeeds, indicating installation."""
        result = class_client.execute("ntpd --version")
        assert 0 == result.return_code

    def test_dist_config_file_is_empty(self, class_client):
        """Test that the distributed config file is empty.

        (This test is skipped on all currently supported Ubuntu releases, so
        may not actually be needed any longer.)
        """
        if class_client.execute("test -e /etc/ntp.conf.dist").failed:
            pytest.skip("/etc/ntp.conf.dist does not exist")
        dist_file = class_client.read_from_file("/etc/ntp.conf.dist")
        assert 0 == len(dist_file.strip().splitlines())

    def test_ntp_entries(self, class_client):
        ntp_conf = class_client.read_from_file("/etc/ntp.conf")
        for expected_server in EXPECTED_SERVERS:
            assert re.search(
                r"^server {} iburst".format(expected_server),
                ntp_conf,
                re.MULTILINE
            )

    def test_ntpq_servers(self, class_client):
        result = class_client.execute("ntpq -p -w -n")
        assert result.ok
        for expected_server in EXPECTED_SERVERS:
            assert expected_server in result.stdout



import os
from cloudinit.analyze.__main__ import (analyze_boot, get_parser)
from cloudinit.tests.helpers import CiTestCase, mock
from cloudinit.analyze.show import dist_check_timestamp, SystemctlReader, \
    FAIL_CODE, CONTAINER_CODE

err_code = (FAIL_CODE, -1, -1, -1)


class TestDistroChecker(CiTestCase):

    @mock.patch('cloudinit.util.system_info', return_value={'dist': ('', '',
                                                                     ''),
                                                            'system': ''})
    @mock.patch('cloudinit.util.get_linux_distro', return_value=('', '', ''))
    @mock.patch('cloudinit.util.is_FreeBSD', return_value=False)
    def test_blank_distro(self, m_sys_info, m_get_linux_distro, m_free_bsd):
        self.assertEqual(err_code, dist_check_timestamp())

    @mock.patch('cloudinit.util.system_info', return_value={'dist': ('', '',
                                                                     '')})
    @mock.patch('cloudinit.util.get_linux_distro', return_value=('', '', ''))
    @mock.patch('cloudinit.util.is_FreeBSD', return_value=True)
    def test_freebsd_gentoo_cant_find(self, m_sys_info,
                                      m_get_linux_distro, m_is_FreeBSD):
        self.assertEqual(err_code, dist_check_timestamp())

    @mock.patch('cloudinit.subp.subp', return_value=(0, 1))
    def test_subp_fails(self, m_subp):
        self.assertEqual(err_code, dist_check_timestamp())


class TestSystemCtlReader(CiTestCase):

    def test_systemctl_invalid_property(self):
        reader = SystemctlReader('dummyProperty')
        with self.assertRaises(RuntimeError):
            reader.parse_epoch_as_float()

    def test_systemctl_invalid_parameter(self):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        with self.assertRaises(RuntimeError):
            reader.parse_epoch_as_float()

    @mock.patch('cloudinit.subp.subp', return_value=('U=1000000', None))
    def test_systemctl_works_correctly_threshold(self, m_subp):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        self.assertEqual(1.0, reader.parse_epoch_as_float())
        thresh = 1.0 - reader.parse_epoch_as_float()
        self.assertTrue(thresh < 1e-6)
        self.assertTrue(thresh > (-1 * 1e-6))

    @mock.patch('cloudinit.subp.subp', return_value=('U=0', None))
    def test_systemctl_succeed_zero(self, m_subp):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        self.assertEqual(0.0, reader.parse_epoch_as_float())

    @mock.patch('cloudinit.subp.subp', return_value=('U=1', None))
    def test_systemctl_succeed_distinct(self, m_subp):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        val1 = reader.parse_epoch_as_float()
        m_subp.return_value = ('U=2', None)
        reader2 = SystemctlReader('dummyProperty', 'dummyParameter')
        val2 = reader2.parse_epoch_as_float()
        self.assertNotEqual(val1, val2)

    @mock.patch('cloudinit.subp.subp', return_value=('100', None))
    def test_systemctl_epoch_not_splittable(self, m_subp):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        with self.assertRaises(IndexError):
            reader.parse_epoch_as_float()

    @mock.patch('cloudinit.subp.subp', return_value=('U=foobar', None))
    def test_systemctl_cannot_convert_epoch_to_float(self, m_subp):
        reader = SystemctlReader('dummyProperty', 'dummyParameter')
        with self.assertRaises(ValueError):
            reader.parse_epoch_as_float()


class TestAnalyzeBoot(CiTestCase):

    def set_up_dummy_file_ci(self, path, log_path):
        infh = open(path, 'w+')
        infh.write('2019-07-08 17:40:49,601 - util.py[DEBUG]: Cloud-init v. '
                   '19.1-1-gbaa47854-0ubuntu1~18.04.1 running \'init-local\' '
                   'at Mon, 08 Jul 2019 17:40:49 +0000. Up 18.84 seconds.')
        infh.close()
        outfh = open(log_path, 'w+')
        outfh.close()

    def set_up_dummy_file(self, path, log_path):
        infh = open(path, 'w+')
        infh.write('dummy data')
        infh.close()
        outfh = open(log_path, 'w+')
        outfh.close()

    def remove_dummy_file(self, path, log_path):
        if os.path.isfile(path):
            os.remove(path)
        if os.path.isfile(log_path):
            os.remove(log_path)

    @mock.patch('cloudinit.analyze.show.dist_check_timestamp',
                return_value=err_code)
    def test_boot_invalid_distro(self, m_dist_check_timestamp):

        path = os.path.dirname(os.path.abspath(__file__))
        log_path = path + '/boot-test.log'
        path += '/dummy.log'
        self.set_up_dummy_file(path, log_path)

        parser = get_parser()
        args = parser.parse_args(args=['boot', '-i', path, '-o',
                                       log_path])
        name_default = ''
        analyze_boot(name_default, args)
        # now args have been tested, go into outfile and make sure error
        # message is in the outfile
        outfh = open(args.outfile, 'r')
        data = outfh.read()
        err_string = 'Your Linux distro or container does not support this ' \
                     'functionality.\nYou must be running a Kernel ' \
                     'Telemetry supported distro.\nPlease check ' \
                     'https://cloudinit.readthedocs.io/en/latest/topics' \
                     '/analyze.html for more information on supported ' \
                     'distros.\n'

        self.remove_dummy_file(path, log_path)
        self.assertEqual(err_string, data)

    @mock.patch("cloudinit.util.is_container", return_value=True)
    @mock.patch('cloudinit.subp.subp', return_value=('U=1000000', None))
    def test_container_no_ci_log_line(self, m_is_container, m_subp):
        path = os.path.dirname(os.path.abspath(__file__))
        log_path = path + '/boot-test.log'
        path += '/dummy.log'
        self.set_up_dummy_file(path, log_path)

        parser = get_parser()
        args = parser.parse_args(args=['boot', '-i', path, '-o',
                                       log_path])
        name_default = ''

        finish_code = analyze_boot(name_default, args)

        self.remove_dummy_file(path, log_path)
        self.assertEqual(FAIL_CODE, finish_code)

    @mock.patch("cloudinit.util.is_container", return_value=True)
    @mock.patch('cloudinit.subp.subp', return_value=('U=1000000', None))
    @mock.patch('cloudinit.analyze.__main__._get_events', return_value=[{
        'name': 'init-local', 'description': 'starting search', 'timestamp':
        100000}])
    @mock.patch('cloudinit.analyze.show.dist_check_timestamp',
                return_value=(CONTAINER_CODE, 1, 1, 1))
    def test_container_ci_log_line(self, m_is_container, m_subp, m_get, m_g):
        path = os.path.dirname(os.path.abspath(__file__))
        log_path = path + '/boot-test.log'
        path += '/dummy.log'
        self.set_up_dummy_file_ci(path, log_path)

        parser = get_parser()
        args = parser.parse_args(args=['boot', '-i', path, '-o',
                                       log_path])
        name_default = ''
        finish_code = analyze_boot(name_default, args)

        self.remove_dummy_file(path, log_path)
        self.assertEqual(CONTAINER_CODE, finish_code)



# This file is part of cloud-init. See LICENSE file for license information.

import copy
import inspect
import os
import stat

from cloudinit.event import EventType
from cloudinit.helpers import Paths
from cloudinit import importer
from cloudinit.sources import (
    EXPERIMENTAL_TEXT, INSTANCE_JSON_FILE, INSTANCE_JSON_SENSITIVE_FILE,
    METADATA_UNKNOWN, REDACT_SENSITIVE_VALUE, UNSET, DataSource,
    canonical_cloud_id, redact_sensitive_keys)
from cloudinit.tests.helpers import CiTestCase, mock
from cloudinit.user_data import UserDataProcessor
from cloudinit import util


class DataSourceTestSubclassNet(DataSource):

    dsname = 'MyTestSubclass'
    url_max_wait = 55

    def __init__(self, sys_cfg, distro, paths, custom_metadata=None,
                 custom_userdata=None, get_data_retval=True):
        super(DataSourceTestSubclassNet, self).__init__(
            sys_cfg, distro, paths)
        self._custom_userdata = custom_userdata
        self._custom_metadata = custom_metadata
        self._get_data_retval = get_data_retval

    def _get_cloud_name(self):
        return 'SubclassCloudName'

    def _get_data(self):
        if self._custom_metadata:
            self.metadata = self._custom_metadata
        else:
            self.metadata = {'availability_zone': 'myaz',
                             'local-hostname': 'test-subclass-hostname',
                             'region': 'myregion'}
        if self._custom_userdata:
            self.userdata_raw = self._custom_userdata
        else:
            self.userdata_raw = 'userdata_raw'
        self.vendordata_raw = 'vendordata_raw'
        return self._get_data_retval


class InvalidDataSourceTestSubclassNet(DataSource):
    pass


class TestDataSource(CiTestCase):

    with_logs = True
    maxDiff = None

    def setUp(self):
        super(TestDataSource, self).setUp()
        self.sys_cfg = {'datasource': {'_undef': {'key1': False}}}
        self.distro = 'distrotest'  # generally should be a Distro object
        self.paths = Paths({})
        self.datasource = DataSource(self.sys_cfg, self.distro, self.paths)

    def test_datasource_init(self):
        """DataSource initializes metadata attributes, ds_cfg and ud_proc."""
        self.assertEqual(self.paths, self.datasource.paths)
        self.assertEqual(self.sys_cfg, self.datasource.sys_cfg)
        self.assertEqual(self.distro, self.datasource.distro)
        self.assertIsNone(self.datasource.userdata)
        self.assertEqual({}, self.datasource.metadata)
        self.assertIsNone(self.datasource.userdata_raw)
        self.assertIsNone(self.datasource.vendordata)
        self.assertIsNone(self.datasource.vendordata_raw)
        self.assertEqual({'key1': False}, self.datasource.ds_cfg)
        self.assertIsInstance(self.datasource.ud_proc, UserDataProcessor)

    def test_datasource_init_gets_ds_cfg_using_dsname(self):
        """Init uses DataSource.dsname for sourcing ds_cfg."""
        sys_cfg = {'datasource': {'MyTestSubclass': {'key2': False}}}
        distro = 'distrotest'  # generally should be a Distro object
        datasource = DataSourceTestSubclassNet(sys_cfg, distro, self.paths)
        self.assertEqual({'key2': False}, datasource.ds_cfg)

    def test_str_is_classname(self):
        """The string representation of the datasource is the classname."""
        self.assertEqual('DataSource', str(self.datasource))
        self.assertEqual(
            'DataSourceTestSubclassNet',
            str(DataSourceTestSubclassNet('', '', self.paths)))

    def test_datasource_get_url_params_defaults(self):
        """get_url_params default url config settings for the datasource."""
        params = self.datasource.get_url_params()
        self.assertEqual(params.max_wait_seconds, self.datasource.url_max_wait)
        self.assertEqual(params.timeout_seconds, self.datasource.url_timeout)
        self.assertEqual(params.num_retries, self.datasource.url_retries)

    def test_datasource_get_url_params_subclassed(self):
        """Subclasses can override get_url_params defaults."""
        sys_cfg = {'datasource': {'MyTestSubclass': {'key2': False}}}
        distro = 'distrotest'  # generally should be a Distro object
        datasource = DataSourceTestSubclassNet(sys_cfg, distro, self.paths)
        expected = (datasource.url_max_wait, datasource.url_timeout,
                    datasource.url_retries)
        url_params = datasource.get_url_params()
        self.assertNotEqual(self.datasource.get_url_params(), url_params)
        self.assertEqual(expected, url_params)

    def test_datasource_get_url_params_ds_config_override(self):
        """Datasource configuration options can override url param defaults."""
        sys_cfg = {
            'datasource': {
                'MyTestSubclass': {
                    'max_wait': '1', 'timeout': '2', 'retries': '3'}}}
        datasource = DataSourceTestSubclassNet(
            sys_cfg, self.distro, self.paths)
        expected = (1, 2, 3)
        url_params = datasource.get_url_params()
        self.assertNotEqual(
            (datasource.url_max_wait, datasource.url_timeout,
             datasource.url_retries),
            url_params)
        self.assertEqual(expected, url_params)

    def test_datasource_get_url_params_is_zero_or_greater(self):
        """get_url_params ignores timeouts with a value below 0."""
        # Set an override that is below 0 which gets ignored.
        sys_cfg = {'datasource': {'_undef': {'timeout': '-1'}}}
        datasource = DataSource(sys_cfg, self.distro, self.paths)
        (_max_wait, timeout, _retries) = datasource.get_url_params()
        self.assertEqual(0, timeout)

    def test_datasource_get_url_uses_defaults_on_errors(self):
        """On invalid system config values for url_params defaults are used."""
        # All invalid values should be logged
        sys_cfg = {'datasource': {
            '_undef': {
                'max_wait': 'nope', 'timeout': 'bug', 'retries': 'nonint'}}}
        datasource = DataSource(sys_cfg, self.distro, self.paths)
        url_params = datasource.get_url_params()
        expected = (datasource.url_max_wait, datasource.url_timeout,
                    datasource.url_retries)
        self.assertEqual(expected, url_params)
        logs = self.logs.getvalue()
        expected_logs = [
            "Config max_wait 'nope' is not an int, using default '-1'",
            "Config timeout 'bug' is not an int, using default '10'",
            "Config retries 'nonint' is not an int, using default '5'",
        ]
        for log in expected_logs:
            self.assertIn(log, logs)

    @mock.patch('cloudinit.sources.net.find_fallback_nic')
    def test_fallback_interface_is_discovered(self, m_get_fallback_nic):
        """The fallback_interface is discovered via find_fallback_nic."""
        m_get_fallback_nic.return_value = 'nic9'
        self.assertEqual('nic9', self.datasource.fallback_interface)

    @mock.patch('cloudinit.sources.net.find_fallback_nic')
    def test_fallback_interface_logs_undiscovered(self, m_get_fallback_nic):
        """Log a warning when fallback_interface can not discover the nic."""
        self.datasource._cloud_name = 'MySupahCloud'
        m_get_fallback_nic.return_value = None  # Couldn't discover nic
        self.assertIsNone(self.datasource.fallback_interface)
        self.assertEqual(
            'WARNING: Did not find a fallback interface on MySupahCloud.\n',
            self.logs.getvalue())

    @mock.patch('cloudinit.sources.net.find_fallback_nic')
    def test_wb_fallback_interface_is_cached(self, m_get_fallback_nic):
        """The fallback_interface is cached and won't be rediscovered."""
        self.datasource._fallback_interface = 'nic10'
        self.assertEqual('nic10', self.datasource.fallback_interface)
        m_get_fallback_nic.assert_not_called()

    def test__get_data_unimplemented(self):
        """Raise an error when _get_data is not implemented."""
        with self.assertRaises(NotImplementedError) as context_manager:
            self.datasource.get_data()
        self.assertIn(
            'Subclasses of DataSource must implement _get_data',
            str(context_manager.exception))
        datasource2 = InvalidDataSourceTestSubclassNet(
            self.sys_cfg, self.distro, self.paths)
        with self.assertRaises(NotImplementedError) as context_manager:
            datasource2.get_data()
        self.assertIn(
            'Subclasses of DataSource must implement _get_data',
            str(context_manager.exception))

    def test_get_data_calls_subclass__get_data(self):
        """Datasource.get_data uses the subclass' version of _get_data."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertTrue(datasource.get_data())
        self.assertEqual(
            {'availability_zone': 'myaz',
             'local-hostname': 'test-subclass-hostname',
             'region': 'myregion'},
            datasource.metadata)
        self.assertEqual('userdata_raw', datasource.userdata_raw)
        self.assertEqual('vendordata_raw', datasource.vendordata_raw)

    def test_get_hostname_strips_local_hostname_without_domain(self):
        """Datasource.get_hostname strips metadata local-hostname of domain."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertTrue(datasource.get_data())
        self.assertEqual(
            'test-subclass-hostname', datasource.metadata['local-hostname'])
        self.assertEqual('test-subclass-hostname', datasource.get_hostname())
        datasource.metadata['local-hostname'] = 'hostname.my.domain.com'
        self.assertEqual('hostname', datasource.get_hostname())

    def test_get_hostname_with_fqdn_returns_local_hostname_with_domain(self):
        """Datasource.get_hostname with fqdn set gets qualified hostname."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertTrue(datasource.get_data())
        datasource.metadata['local-hostname'] = 'hostname.my.domain.com'
        self.assertEqual(
            'hostname.my.domain.com', datasource.get_hostname(fqdn=True))

    def test_get_hostname_without_metadata_uses_system_hostname(self):
        """Datasource.gethostname runs util.get_hostname when no metadata."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertEqual({}, datasource.metadata)
        mock_fqdn = 'cloudinit.sources.util.get_fqdn_from_hosts'
        with mock.patch('cloudinit.sources.util.get_hostname') as m_gethost:
            with mock.patch(mock_fqdn) as m_fqdn:
                m_gethost.return_value = 'systemhostname.domain.com'
                m_fqdn.return_value = None  # No maching fqdn in /etc/hosts
                self.assertEqual('systemhostname', datasource.get_hostname())
                self.assertEqual(
                    'systemhostname.domain.com',
                    datasource.get_hostname(fqdn=True))

    def test_get_hostname_without_metadata_returns_none(self):
        """Datasource.gethostname returns None when metadata_only and no MD."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertEqual({}, datasource.metadata)
        mock_fqdn = 'cloudinit.sources.util.get_fqdn_from_hosts'
        with mock.patch('cloudinit.sources.util.get_hostname') as m_gethost:
            with mock.patch(mock_fqdn) as m_fqdn:
                self.assertIsNone(datasource.get_hostname(metadata_only=True))
                self.assertIsNone(
                    datasource.get_hostname(fqdn=True, metadata_only=True))
        self.assertEqual([], m_gethost.call_args_list)
        self.assertEqual([], m_fqdn.call_args_list)

    def test_get_hostname_without_metadata_prefers_etc_hosts(self):
        """Datasource.gethostname prefers /etc/hosts to util.get_hostname."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        self.assertEqual({}, datasource.metadata)
        mock_fqdn = 'cloudinit.sources.util.get_fqdn_from_hosts'
        with mock.patch('cloudinit.sources.util.get_hostname') as m_gethost:
            with mock.patch(mock_fqdn) as m_fqdn:
                m_gethost.return_value = 'systemhostname.domain.com'
                m_fqdn.return_value = 'fqdnhostname.domain.com'
                self.assertEqual('fqdnhostname', datasource.get_hostname())
                self.assertEqual('fqdnhostname.domain.com',
                                 datasource.get_hostname(fqdn=True))

    def test_get_data_does_not_write_instance_data_on_failure(self):
        """get_data does not write INSTANCE_JSON_FILE on get_data False."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}),
            get_data_retval=False)
        self.assertFalse(datasource.get_data())
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        self.assertFalse(
            os.path.exists(json_file), 'Found unexpected file %s' % json_file)

    def test_get_data_writes_json_instance_data_on_success(self):
        """get_data writes INSTANCE_JSON_FILE to run_dir as world readable."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        sys_info = {
            "python": "3.7",
            "platform":
                "Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal",
            "uname": ["Linux", "myhost", "5.4.0-24-generic", "SMP blah",
                      "x86_64"],
            "variant": "ubuntu", "dist": ["ubuntu", "20.04", "focal"]}
        with mock.patch("cloudinit.util.system_info", return_value=sys_info):
            datasource.get_data()
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        content = util.load_file(json_file)
        expected = {
            'base64_encoded_keys': [],
            'merged_cfg': REDACT_SENSITIVE_VALUE,
            'sensitive_keys': ['merged_cfg'],
            'sys_info': sys_info,
            'v1': {
                '_beta_keys': ['subplatform'],
                'availability-zone': 'myaz',
                'availability_zone': 'myaz',
                'cloud-name': 'subclasscloudname',
                'cloud_name': 'subclasscloudname',
                'distro': 'ubuntu',
                'distro_release': 'focal',
                'distro_version': '20.04',
                'instance-id': 'iid-datasource',
                'instance_id': 'iid-datasource',
                'local-hostname': 'test-subclass-hostname',
                'local_hostname': 'test-subclass-hostname',
                'kernel_release': '5.4.0-24-generic',
                'machine': 'x86_64',
                'platform': 'mytestsubclass',
                'public_ssh_keys': [],
                'python_version': '3.7',
                'region': 'myregion',
                'system_platform':
                    'Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal',
                'subplatform': 'unknown',
                'variant': 'ubuntu'},
            'ds': {

                '_doc': EXPERIMENTAL_TEXT,
                'meta_data': {'availability_zone': 'myaz',
                              'local-hostname': 'test-subclass-hostname',
                              'region': 'myregion'}}}
        self.assertEqual(expected, util.load_json(content))
        file_stat = os.stat(json_file)
        self.assertEqual(0o644, stat.S_IMODE(file_stat.st_mode))
        self.assertEqual(expected, util.load_json(content))

    def test_get_data_writes_redacted_public_json_instance_data(self):
        """get_data writes redacted content to public INSTANCE_JSON_FILE."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}),
            custom_metadata={
                'availability_zone': 'myaz',
                'local-hostname': 'test-subclass-hostname',
                'region': 'myregion',
                'some': {'security-credentials': {
                    'cred1': 'sekret', 'cred2': 'othersekret'}}})
        self.assertCountEqual(
            ('merged_cfg', 'security-credentials',),
            datasource.sensitive_metadata_keys)
        sys_info = {
            "python": "3.7",
            "platform":
                "Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal",
            "uname": ["Linux", "myhost", "5.4.0-24-generic", "SMP blah",
                      "x86_64"],
            "variant": "ubuntu", "dist": ["ubuntu", "20.04", "focal"]}
        with mock.patch("cloudinit.util.system_info", return_value=sys_info):
            datasource.get_data()
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        redacted = util.load_json(util.load_file(json_file))
        expected = {
            'base64_encoded_keys': [],
            'merged_cfg': REDACT_SENSITIVE_VALUE,
            'sensitive_keys': [
                'ds/meta_data/some/security-credentials', 'merged_cfg'],
            'sys_info': sys_info,
            'v1': {
                '_beta_keys': ['subplatform'],
                'availability-zone': 'myaz',
                'availability_zone': 'myaz',
                'cloud-name': 'subclasscloudname',
                'cloud_name': 'subclasscloudname',
                'distro': 'ubuntu',
                'distro_release': 'focal',
                'distro_version': '20.04',
                'instance-id': 'iid-datasource',
                'instance_id': 'iid-datasource',
                'local-hostname': 'test-subclass-hostname',
                'local_hostname': 'test-subclass-hostname',
                'kernel_release': '5.4.0-24-generic',
                'machine': 'x86_64',
                'platform': 'mytestsubclass',
                'public_ssh_keys': [],
                'python_version': '3.7',
                'region': 'myregion',
                'system_platform':
                    'Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal',
                'subplatform': 'unknown',
                'variant': 'ubuntu'},
            'ds': {
                '_doc': EXPERIMENTAL_TEXT,
                'meta_data': {
                    'availability_zone': 'myaz',
                    'local-hostname': 'test-subclass-hostname',
                    'region': 'myregion',
                    'some': {'security-credentials': REDACT_SENSITIVE_VALUE}}}
        }
        self.assertCountEqual(expected, redacted)
        file_stat = os.stat(json_file)
        self.assertEqual(0o644, stat.S_IMODE(file_stat.st_mode))

    def test_get_data_writes_json_instance_data_sensitive(self):
        """
        get_data writes unmodified data to sensitive file as root-readonly.
        """
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}),
            custom_metadata={
                'availability_zone': 'myaz',
                'local-hostname': 'test-subclass-hostname',
                'region': 'myregion',
                'some': {'security-credentials': {
                    'cred1': 'sekret', 'cred2': 'othersekret'}}})
        sys_info = {
            "python": "3.7",
            "platform":
                "Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal",
            "uname": ["Linux", "myhost", "5.4.0-24-generic", "SMP blah",
                      "x86_64"],
            "variant": "ubuntu", "dist": ["ubuntu", "20.04", "focal"]}

        self.assertCountEqual(
            ('merged_cfg', 'security-credentials',),
            datasource.sensitive_metadata_keys)
        with mock.patch("cloudinit.util.system_info", return_value=sys_info):
            datasource.get_data()
        sensitive_json_file = self.tmp_path(INSTANCE_JSON_SENSITIVE_FILE, tmp)
        content = util.load_file(sensitive_json_file)
        expected = {
            'base64_encoded_keys': [],
            'merged_cfg': {
                '_doc': (
                    'Merged cloud-init system config from '
                    '/etc/cloud/cloud.cfg and /etc/cloud/cloud.cfg.d/'
                ),
                'datasource': {'_undef': {'key1': False}}},
            'sensitive_keys': [
                'ds/meta_data/some/security-credentials', 'merged_cfg'],
            'sys_info': sys_info,
            'v1': {
                '_beta_keys': ['subplatform'],
                'availability-zone': 'myaz',
                'availability_zone': 'myaz',
                'cloud-name': 'subclasscloudname',
                'cloud_name': 'subclasscloudname',
                'distro': 'ubuntu',
                'distro_release': 'focal',
                'distro_version': '20.04',
                'instance-id': 'iid-datasource',
                'instance_id': 'iid-datasource',
                'kernel_release': '5.4.0-24-generic',
                'local-hostname': 'test-subclass-hostname',
                'local_hostname': 'test-subclass-hostname',
                'machine': 'x86_64',
                'platform': 'mytestsubclass',
                'public_ssh_keys': [],
                'python_version': '3.7',
                'region': 'myregion',
                'subplatform': 'unknown',
                'system_platform':
                    'Linux-5.4.0-24-generic-x86_64-with-Ubuntu-20.04-focal',
                'variant': 'ubuntu'},
            'ds': {
                '_doc': EXPERIMENTAL_TEXT,
                'meta_data': {
                    'availability_zone': 'myaz',
                    'local-hostname': 'test-subclass-hostname',
                    'region': 'myregion',
                    'some': {
                        'security-credentials':
                            {'cred1': 'sekret', 'cred2': 'othersekret'}}}}
        }
        self.assertCountEqual(expected, util.load_json(content))
        file_stat = os.stat(sensitive_json_file)
        self.assertEqual(0o600, stat.S_IMODE(file_stat.st_mode))
        self.assertEqual(expected, util.load_json(content))

    def test_get_data_handles_redacted_unserializable_content(self):
        """get_data warns unserializable content in INSTANCE_JSON_FILE."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}),
            custom_metadata={'key1': 'val1', 'key2': {'key2.1': self.paths}})
        datasource.get_data()
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        content = util.load_file(json_file)
        expected_metadata = {
            'key1': 'val1',
            'key2': {
                'key2.1': "Warning: redacted unserializable type <class"
                          " 'cloudinit.helpers.Paths'>"}}
        instance_json = util.load_json(content)
        self.assertEqual(
            expected_metadata, instance_json['ds']['meta_data'])

    def test_persist_instance_data_writes_ec2_metadata_when_set(self):
        """When ec2_metadata class attribute is set, persist to json."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        datasource.ec2_metadata = UNSET
        datasource.get_data()
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        instance_data = util.load_json(util.load_file(json_file))
        self.assertNotIn('ec2_metadata', instance_data['ds'])
        datasource.ec2_metadata = {'ec2stuff': 'is good'}
        datasource.persist_instance_data()
        instance_data = util.load_json(util.load_file(json_file))
        self.assertEqual(
            {'ec2stuff': 'is good'},
            instance_data['ds']['ec2_metadata'])

    def test_persist_instance_data_writes_network_json_when_set(self):
        """When network_data.json class attribute is set, persist to json."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}))
        datasource.get_data()
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        instance_data = util.load_json(util.load_file(json_file))
        self.assertNotIn('network_json', instance_data['ds'])
        datasource.network_json = {'network_json': 'is good'}
        datasource.persist_instance_data()
        instance_data = util.load_json(util.load_file(json_file))
        self.assertEqual(
            {'network_json': 'is good'},
            instance_data['ds']['network_json'])

    def test_get_data_base64encodes_unserializable_bytes(self):
        """On py3, get_data base64encodes any unserializable content."""
        tmp = self.tmp_dir()
        datasource = DataSourceTestSubclassNet(
            self.sys_cfg, self.distro, Paths({'run_dir': tmp}),
            custom_metadata={'key1': 'val1', 'key2': {'key2.1': b'\x123'}})
        self.assertTrue(datasource.get_data())
        json_file = self.tmp_path(INSTANCE_JSON_FILE, tmp)
        content = util.load_file(json_file)
        instance_json = util.load_json(content)
        self.assertCountEqual(
            ['ds/meta_data/key2/key2.1'],
            instance_json['base64_encoded_keys'])
        self.assertEqual(
            {'key1': 'val1', 'key2': {'key2.1': 'EjM='}},
            instance_json['ds']['meta_data'])

    def test_get_hostname_subclass_support(self):
        """Validate get_hostname signature on all subclasses of DataSource."""
        base_args = inspect.getfullargspec(DataSource.get_hostname)
        # Import all DataSource subclasses so we can inspect them.
        modules = util.find_modules(os.path.dirname(os.path.dirname(__file__)))
        for _loc, name in modules.items():
            mod_locs, _ = importer.find_module(name, ['cloudinit.sources'], [])
            if mod_locs:
                importer.import_module(mod_locs[0])
        for child in DataSource.__subclasses__():
            if 'Test' in child.dsname:
                continue
            self.assertEqual(
                base_args,
                inspect.getfullargspec(child.get_hostname),
                '%s does not implement DataSource.get_hostname params'
                % child)
            for grandchild in child.__subclasses__():
                self.assertEqual(
                    base_args,
                    inspect.getfullargspec(grandchild.get_hostname),
                    '%s does not implement DataSource.get_hostname params'
                    % grandchild)

    def test_clear_cached_attrs_resets_cached_attr_class_attributes(self):
        """Class attributes listed in cached_attr_defaults are reset."""
        count = 0
        # Setup values for all cached class attributes
        for attr, value in self.datasource.cached_attr_defaults:
            setattr(self.datasource, attr, count)
            count += 1
        self.datasource._dirty_cache = True
        self.datasource.clear_cached_attrs()
        for attr, value in self.datasource.cached_attr_defaults:
            self.assertEqual(value, getattr(self.datasource, attr))

    def test_clear_cached_attrs_noops_on_clean_cache(self):
        """Class attributes listed in cached_attr_defaults are reset."""
        count = 0
        # Setup values for all cached class attributes
        for attr, _ in self.datasource.cached_attr_defaults:
            setattr(self.datasource, attr, count)
            count += 1
        self.datasource._dirty_cache = False   # Fake clean cache
        self.datasource.clear_cached_attrs()
        count = 0
        for attr, _ in self.datasource.cached_attr_defaults:
            self.assertEqual(count, getattr(self.datasource, attr))
            count += 1

    def test_clear_cached_attrs_skips_non_attr_class_attributes(self):
        """Skip any cached_attr_defaults which aren't class attributes."""
        self.datasource._dirty_cache = True
        self.datasource.clear_cached_attrs()
        for attr in ('ec2_metadata', 'network_json'):
            self.assertFalse(hasattr(self.datasource, attr))

    def test_clear_cached_attrs_of_custom_attrs(self):
        """Custom attr_values can be passed to clear_cached_attrs."""
        self.datasource._dirty_cache = True
        cached_attr_name = self.datasource.cached_attr_defaults[0][0]
        setattr(self.datasource, cached_attr_name, 'himom')
        self.datasource.myattr = 'orig'
        self.datasource.clear_cached_attrs(
            attr_defaults=(('myattr', 'updated'),))
        self.assertEqual('himom', getattr(self.datasource, cached_attr_name))
        self.assertEqual('updated', self.datasource.myattr)

    def test_update_metadata_only_acts_on_supported_update_events(self):
        """update_metadata won't get_data on unsupported update events."""
        self.datasource.update_events['network'].discard(EventType.BOOT)
        self.assertEqual(
            {'network': set([EventType.BOOT_NEW_INSTANCE])},
            self.datasource.update_events)

        def fake_get_data():
            raise Exception('get_data should not be called')

        self.datasource.get_data = fake_get_data
        self.assertFalse(
            self.datasource.update_metadata(
                source_event_types=[EventType.BOOT]))

    def test_update_metadata_returns_true_on_supported_update_event(self):
        """update_metadata returns get_data response on supported events."""

        def fake_get_data():
            return True

        self.datasource.get_data = fake_get_data
        self.datasource._network_config = 'something'
        self.datasource._dirty_cache = True
        self.assertTrue(
            self.datasource.update_metadata(
                source_event_types=[
                    EventType.BOOT, EventType.BOOT_NEW_INSTANCE]))
        self.assertEqual(UNSET, self.datasource._network_config)
        self.assertIn(
            "DEBUG: Update datasource metadata and network config due to"
            " events: New instance first boot",
            self.logs.getvalue())


class TestRedactSensitiveData(CiTestCase):

    def test_redact_sensitive_data_noop_when_no_sensitive_keys_present(self):
        """When sensitive_keys is absent or empty from metadata do nothing."""
        md = {'my': 'data'}
        self.assertEqual(
            md, redact_sensitive_keys(md, redact_value='redacted'))
        md['sensitive_keys'] = []
        self.assertEqual(
            md, redact_sensitive_keys(md, redact_value='redacted'))

    def test_redact_sensitive_data_redacts_exact_match_name(self):
        """Only exact matched sensitive_keys are redacted from metadata."""
        md = {'sensitive_keys': ['md/secure'],
              'md': {'secure': 's3kr1t', 'insecure': 'publik'}}
        secure_md = copy.deepcopy(md)
        secure_md['md']['secure'] = 'redacted'
        self.assertEqual(
            secure_md,
            redact_sensitive_keys(md, redact_value='redacted'))

    def test_redact_sensitive_data_does_redacts_with_default_string(self):
        """When redact_value is absent, REDACT_SENSITIVE_VALUE is used."""
        md = {'sensitive_keys': ['md/secure'],
              'md': {'secure': 's3kr1t', 'insecure': 'publik'}}
        secure_md = copy.deepcopy(md)
        secure_md['md']['secure'] = 'redacted for non-root user'
        self.assertEqual(
            secure_md,
            redact_sensitive_keys(md))


class TestCanonicalCloudID(CiTestCase):

    def test_cloud_id_returns_platform_on_unknowns(self):
        """When region and cloud_name are unknown, return platform."""
        self.assertEqual(
            'platform',
            canonical_cloud_id(cloud_name=METADATA_UNKNOWN,
                               region=METADATA_UNKNOWN,
                               platform='platform'))

    def test_cloud_id_returns_platform_on_none(self):
        """When region and cloud_name are unknown, return platform."""
        self.assertEqual(
            'platform',
            canonical_cloud_id(cloud_name=None,
                               region=None,
                               platform='platform'))

    def test_cloud_id_returns_cloud_name_on_unknown_region(self):
        """When region is unknown, return cloud_name."""
        for region in (None, METADATA_UNKNOWN):
            self.assertEqual(
                'cloudname',
                canonical_cloud_id(cloud_name='cloudname',
                                   region=region,
                                   platform='platform'))

    def test_cloud_id_returns_platform_on_unknown_cloud_name(self):
        """When region is set but cloud_name is unknown return cloud_name."""
        self.assertEqual(
            'platform',
            canonical_cloud_id(cloud_name=METADATA_UNKNOWN,
                               region='region',
                               platform='platform'))

    def test_cloud_id_aws_based_on_region_and_cloud_name(self):
        """When cloud_name is aws, return proper cloud-id based on region."""
        self.assertEqual(
            'aws-china',
            canonical_cloud_id(cloud_name='aws',
                               region='cn-north-1',
                               platform='platform'))
        self.assertEqual(
            'aws',
            canonical_cloud_id(cloud_name='aws',
                               region='us-east-1',
                               platform='platform'))
        self.assertEqual(
            'aws-gov',
            canonical_cloud_id(cloud_name='aws',
                               region='us-gov-1',
                               platform='platform'))
        self.assertEqual(  # Overrideen non-aws cloud_name is returned
            '!aws',
            canonical_cloud_id(cloud_name='!aws',
                               region='us-gov-1',
                               platform='platform'))

    def test_cloud_id_azure_based_on_region_and_cloud_name(self):
        """Report cloud-id when cloud_name is azure and region is in china."""
        self.assertEqual(
            'azure-china',
            canonical_cloud_id(cloud_name='azure',
                               region='chinaeast',
                               platform='platform'))
        self.assertEqual(
            'azure',
            canonical_cloud_id(cloud_name='azure',
                               region='!chinaeast',
                               platform='platform'))

# vi: ts=4 expandtab



# Copyright (C) 2019-2020 Gonéri Le Bouder
#
# This file is part of cloud-init. See LICENSE file for license information.

import crypt
import os
import platform

import cloudinit.distros.bsd
from cloudinit import log as logging
from cloudinit import subp
from cloudinit import util

LOG = logging.getLogger(__name__)


class NetBSD(cloudinit.distros.bsd.BSD):
    """
    Distro subclass for NetBSD.

    (N.B. OpenBSD inherits from this class.)
    """

    ci_sudoers_fn = '/usr/pkg/etc/sudoers.d/90-cloud-init-users'
    group_add_cmd_prefix = ["groupadd"]

    def __init__(self, name, cfg, paths):
        super().__init__(name, cfg, paths)
        if os.path.exists("/usr/pkg/bin/pkgin"):
            self.pkg_cmd_install_prefix = ['pkgin', '-y', 'install']
            self.pkg_cmd_remove_prefix = ['pkgin', '-y', 'remove']
            self.pkg_cmd_update_prefix = ['pkgin', '-y', 'update']
            self.pkg_cmd_upgrade_prefix = ['pkgin', '-y', 'full-upgrade']
        else:
            self.pkg_cmd_install_prefix = ['pkg_add', '-U']
            self.pkg_cmd_remove_prefix = ['pkg_delete']

    def _get_add_member_to_group_cmd(self, member_name, group_name):
        return ['usermod', '-G', group_name, member_name]

    def add_user(self, name, **kwargs):
        if util.is_user(name):
            LOG.info("User %s already exists, skipping.", name)
            return False

        adduser_cmd = ['useradd']
        log_adduser_cmd = ['useradd']

        adduser_opts = {
            "homedir": '-d',
            "gecos": '-c',
            "primary_group": '-g',
            "groups": '-G',
            "shell": '-s',
        }
        adduser_flags = {
            "no_user_group": '--no-user-group',
            "system": '--system',
            "no_log_init": '--no-log-init',
        }

        for key, val in kwargs.items():
            if key in adduser_opts and val and isinstance(val, str):
                adduser_cmd.extend([adduser_opts[key], val])

            elif key in adduser_flags and val:
                adduser_cmd.append(adduser_flags[key])
                log_adduser_cmd.append(adduser_flags[key])

        if 'no_create_home' not in kwargs or 'system' not in kwargs:
            adduser_cmd += ['-m']
            log_adduser_cmd += ['-m']

        adduser_cmd += [name]
        log_adduser_cmd += [name]

        # Run the command
        LOG.info("Adding user %s", name)
        try:
            subp.subp(adduser_cmd, logstring=log_adduser_cmd)
        except Exception:
            util.logexc(LOG, "Failed to create user %s", name)
            raise
        # Set the password if it is provided
        # For security consideration, only hashed passwd is assumed
        passwd_val = kwargs.get('passwd', None)
        if passwd_val is not None:
            self.set_passwd(name, passwd_val, hashed=True)

    def set_passwd(self, user, passwd, hashed=False):
        if hashed:
            hashed_pw = passwd
        elif not hasattr(crypt, 'METHOD_BLOWFISH'):
            # crypt.METHOD_BLOWFISH comes with Python 3.7 which is available
            # on NetBSD 7 and 8.
            LOG.error((
                'Cannot set non-encrypted password for user %s. '
                'Python >= 3.7 is required.'), user)
            return
        else:
            method = crypt.METHOD_BLOWFISH  # pylint: disable=E1101
            hashed_pw = crypt.crypt(
                passwd,
                crypt.mksalt(method)
            )

        try:
            subp.subp(['usermod', '-p', hashed_pw, user])
        except Exception:
            util.logexc(LOG, "Failed to set password for %s", user)
            raise
        self.unlock_passwd(user)

    def force_passwd_change(self, user):
        try:
            subp.subp(['usermod', '-F', user])
        except Exception:
            util.logexc(LOG, "Failed to set pw expiration for %s", user)
            raise

    def lock_passwd(self, name):
        try:
            subp.subp(['usermod', '-C', 'yes', name])
        except Exception:
            util.logexc(LOG, "Failed to lock user %s", name)
            raise

    def unlock_passwd(self, name):
        try:
            subp.subp(['usermod', '-C', 'no', name])
        except Exception:
            util.logexc(LOG, "Failed to unlock user %s", name)
            raise

    def apply_locale(self, locale, out_fn=None):
        LOG.debug('Cannot set the locale.')

    def apply_network_config_names(self, netconfig):
        LOG.debug('NetBSD cannot rename network interface.')

    def _get_pkg_cmd_environ(self):
        """Return env vars used in NetBSD package_command operations"""
        os_release = platform.release()
        os_arch = platform.machine()
        e = os.environ.copy()
        e['PKG_PATH'] = (
            'http://cdn.netbsd.org/pub/pkgsrc/'
            'packages/NetBSD/%s/%s/All'
        ) % (os_arch, os_release)
        return e

    def update_package_sources(self):
        pass


class Distro(NetBSD):
    pass

# vi: ts=4 expandtab



# This file is part of cloud-init. See LICENSE file for license information.

from cloudinit import helpers
from cloudinit.sources import DataSourceOpenNebula as ds
from cloudinit import util
from cloudinit.tests.helpers import mock, populate_dir, CiTestCase

import os
import pwd
import unittest

import pytest


TEST_VARS = {
    'VAR1': 'single',
    'VAR2': 'double word',
    'VAR3': 'multi\nline\n',
    'VAR4': "'single'",
    'VAR5': "'double word'",
    'VAR6': "'multi\nline\n'",
    'VAR7': 'single\\t',
    'VAR8': 'double\\tword',
    'VAR9': 'multi\\t\nline\n',
    'VAR10': '\\',  # expect '\'
    'VAR11': '\'',  # expect '
    'VAR12': '$',   # expect $
}

INVALID_CONTEXT = ';'
USER_DATA = '#cloud-config\napt_upgrade: true'
SSH_KEY = 'ssh-rsa AAAAB3NzaC1....sIkJhq8wdX+4I3A4cYbYP ubuntu@server-460-%i'
HOSTNAME = 'foo.example.com'
PUBLIC_IP = '10.0.0.3'
MACADDR = '02:00:0a:12:01:01'
IP_BY_MACADDR = '10.18.1.1'
IP4_PREFIX = '24'
IP6_GLOBAL = '2001:db8:1:0:400:c0ff:fea8:1ba'
IP6_ULA = 'fd01:dead:beaf:0:400:c0ff:fea8:1ba'
IP6_GW = '2001:db8:1::ffff'
IP6_PREFIX = '48'

DS_PATH = "cloudinit.sources.DataSourceOpenNebula"


class TestOpenNebulaDataSource(CiTestCase):
    parsed_user = None
    allowed_subp = ['bash']

    def setUp(self):
        super(TestOpenNebulaDataSource, self).setUp()
        self.tmp = self.tmp_dir()
        self.paths = helpers.Paths(
            {'cloud_dir': self.tmp, 'run_dir': self.tmp})

        # defaults for few tests
        self.ds = ds.DataSourceOpenNebula
        self.seed_dir = os.path.join(self.paths.seed_dir, "opennebula")
        self.sys_cfg = {'datasource': {'OpenNebula': {'dsmode': 'local'}}}

        # we don't want 'sudo' called in tests. so we patch switch_user_cmd
        def my_switch_user_cmd(user):
            self.parsed_user = user
            return []

        self.switch_user_cmd_real = ds.switch_user_cmd
        ds.switch_user_cmd = my_switch_user_cmd

    def tearDown(self):
        ds.switch_user_cmd = self.switch_user_cmd_real
        super(TestOpenNebulaDataSource, self).tearDown()

    def test_get_data_non_contextdisk(self):
        orig_find_devs_with = util.find_devs_with
        try:
            # dont' try to lookup for CDs
            util.find_devs_with = lambda n: []
            dsrc = self.ds(sys_cfg=self.sys_cfg, distro=None, paths=self.paths)
            ret = dsrc.get_data()
            self.assertFalse(ret)
        finally:
            util.find_devs_with = orig_find_devs_with

    def test_get_data_broken_contextdisk(self):
        orig_find_devs_with = util.find_devs_with
        try:
            # dont' try to lookup for CDs
            util.find_devs_with = lambda n: []
            populate_dir(self.seed_dir, {'context.sh': INVALID_CONTEXT})
            dsrc = self.ds(sys_cfg=self.sys_cfg, distro=None, paths=self.paths)
            self.assertRaises(ds.BrokenContextDiskDir, dsrc.get_data)
        finally:
            util.find_devs_with = orig_find_devs_with

    def test_get_data_invalid_identity(self):
        orig_find_devs_with = util.find_devs_with
        try:
            # generate non-existing system user name
            sys_cfg = self.sys_cfg
            invalid_user = 'invalid'
            while not sys_cfg['datasource']['OpenNebula'].get('parseuser'):
                try:
                    pwd.getpwnam(invalid_user)
                    invalid_user += 'X'
                except KeyError:
                    sys_cfg['datasource']['OpenNebula']['parseuser'] = \
                        invalid_user

            # dont' try to lookup for CDs
            util.find_devs_with = lambda n: []
            populate_context_dir(self.seed_dir, {'KEY1': 'val1'})
            dsrc = self.ds(sys_cfg=sys_cfg, distro=None, paths=self.paths)
            self.assertRaises(ds.BrokenContextDiskDir, dsrc.get_data)
        finally:
            util.find_devs_with = orig_find_devs_with

    def test_get_data(self):
        orig_find_devs_with = util.find_devs_with
        try:
            # dont' try to lookup for CDs
            util.find_devs_with = lambda n: []
            populate_context_dir(self.seed_dir, {'KEY1': 'val1'})
            dsrc = self.ds(sys_cfg=self.sys_cfg, distro=None, paths=self.paths)
            ret = dsrc.get_data()
            self.assertTrue(ret)
        finally:
            util.find_devs_with = orig_find_devs_with
        self.assertEqual('opennebula', dsrc.cloud_name)
        self.assertEqual('opennebula', dsrc.platform_type)
        self.assertEqual(
            'seed-dir (%s/seed/opennebula)' % self.tmp, dsrc.subplatform)

    def test_seed_dir_non_contextdisk(self):
        self.assertRaises(ds.NonContextDiskDir, ds.read_context_disk_dir,
                          self.seed_dir, mock.Mock())

    def test_seed_dir_empty1_context(self):
        populate_dir(self.seed_dir, {'context.sh': ''})
        results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

        self.assertIsNone(results['userdata'])
        self.assertEqual(results['metadata'], {})

    def test_seed_dir_empty2_context(self):
        populate_context_dir(self.seed_dir, {})
        results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

        self.assertIsNone(results['userdata'])
        self.assertEqual(results['metadata'], {})

    def test_seed_dir_broken_context(self):
        populate_dir(self.seed_dir, {'context.sh': INVALID_CONTEXT})

        self.assertRaises(ds.BrokenContextDiskDir,
                          ds.read_context_disk_dir,
                          self.seed_dir, mock.Mock())

    def test_context_parser(self):
        populate_context_dir(self.seed_dir, TEST_VARS)
        results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

        self.assertTrue('metadata' in results)
        self.assertEqual(TEST_VARS, results['metadata'])

    def test_ssh_key(self):
        public_keys = ['first key', 'second key']
        for c in range(4):
            for k in ('SSH_KEY', 'SSH_PUBLIC_KEY'):
                my_d = os.path.join(self.tmp, "%s-%i" % (k, c))
                populate_context_dir(my_d, {k: '\n'.join(public_keys)})
                results = ds.read_context_disk_dir(my_d, mock.Mock())

                self.assertTrue('metadata' in results)
                self.assertTrue('public-keys' in results['metadata'])
                self.assertEqual(public_keys,
                                 results['metadata']['public-keys'])

            public_keys.append(SSH_KEY % (c + 1,))

    def test_user_data_plain(self):
        for k in ('USER_DATA', 'USERDATA'):
            my_d = os.path.join(self.tmp, k)
            populate_context_dir(my_d, {k: USER_DATA,
                                        'USERDATA_ENCODING': ''})
            results = ds.read_context_disk_dir(my_d, mock.Mock())

            self.assertTrue('userdata' in results)
            self.assertEqual(USER_DATA, results['userdata'])

    def test_user_data_encoding_required_for_decode(self):
        b64userdata = util.b64e(USER_DATA)
        for k in ('USER_DATA', 'USERDATA'):
            my_d = os.path.join(self.tmp, k)
            populate_context_dir(my_d, {k: b64userdata})
            results = ds.read_context_disk_dir(my_d, mock.Mock())

            self.assertTrue('userdata' in results)
            self.assertEqual(b64userdata, results['userdata'])

    def test_user_data_base64_encoding(self):
        for k in ('USER_DATA', 'USERDATA'):
            my_d = os.path.join(self.tmp, k)
            populate_context_dir(my_d, {k: util.b64e(USER_DATA),
                                        'USERDATA_ENCODING': 'base64'})
            results = ds.read_context_disk_dir(my_d, mock.Mock())

            self.assertTrue('userdata' in results)
            self.assertEqual(USER_DATA, results['userdata'])

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_hostname(self, m_get_phys_by_mac):
        for dev in ('eth0', 'ens3'):
            m_get_phys_by_mac.return_value = {MACADDR: dev}
            for k in ('HOSTNAME', 'PUBLIC_IP', 'IP_PUBLIC', 'ETH0_IP'):
                my_d = os.path.join(self.tmp, k)
                populate_context_dir(my_d, {k: PUBLIC_IP})
                results = ds.read_context_disk_dir(my_d, mock.Mock())

                self.assertTrue('metadata' in results)
                self.assertTrue('local-hostname' in results['metadata'])
                self.assertEqual(
                    PUBLIC_IP, results['metadata']['local-hostname'])

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_network_interfaces(self, m_get_phys_by_mac):
        for dev in ('eth0', 'ens3'):
            m_get_phys_by_mac.return_value = {MACADDR: dev}

            # without ETH0_MAC
            # for Older OpenNebula?
            populate_context_dir(self.seed_dir, {'ETH0_IP': IP_BY_MACADDR})
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP_BY_MACADDR + '/' + IP4_PREFIX in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP and ETH0_MAC
            populate_context_dir(
                self.seed_dir, {'ETH0_IP': IP_BY_MACADDR, 'ETH0_MAC': MACADDR})
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP_BY_MACADDR + '/' + IP4_PREFIX in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP with empty string and ETH0_MAC
            # in the case of using Virtual Network contains
            # "AR = [ TYPE = ETHER ]"
            populate_context_dir(
                self.seed_dir, {'ETH0_IP': '', 'ETH0_MAC': MACADDR})
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP_BY_MACADDR + '/' + IP4_PREFIX in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_MASK
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP': IP_BY_MACADDR,
                    'ETH0_MAC': MACADDR,
                    'ETH0_MASK': '255.255.0.0'
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP_BY_MACADDR + '/16' in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_MASK with empty string
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP': IP_BY_MACADDR,
                    'ETH0_MAC': MACADDR,
                    'ETH0_MASK': ''
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP_BY_MACADDR + '/' + IP4_PREFIX in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP6
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP6': IP6_GLOBAL,
                    'ETH0_MAC': MACADDR,
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP6_GLOBAL + '/64' in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP6_ULA
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP6_ULA': IP6_ULA,
                    'ETH0_MAC': MACADDR,
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP6_ULA + '/64' in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP6 and ETH0_IP6_PREFIX_LENGTH
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP6': IP6_GLOBAL,
                    'ETH0_IP6_PREFIX_LENGTH': IP6_PREFIX,
                    'ETH0_MAC': MACADDR,
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP6_GLOBAL + '/' + IP6_PREFIX in
                results['network-interfaces']['ethernets'][dev]['addresses'])

            # ETH0_IP6 and ETH0_IP6_PREFIX_LENGTH with empty string
            populate_context_dir(
                self.seed_dir, {
                    'ETH0_IP6': IP6_GLOBAL,
                    'ETH0_IP6_PREFIX_LENGTH': '',
                    'ETH0_MAC': MACADDR,
                })
            results = ds.read_context_disk_dir(self.seed_dir, mock.Mock())

            self.assertTrue('network-interfaces' in results)
            self.assertTrue(
                IP6_GLOBAL + '/64' in
                results['network-interfaces']['ethernets'][dev]['addresses'])

    def test_find_candidates(self):
        def my_devs_with(criteria):
            return {
                "LABEL=CONTEXT": ["/dev/sdb"],
                "LABEL=CDROM": ["/dev/sr0"],
                "TYPE=iso9660": ["/dev/vdb"],
            }.get(criteria, [])

        orig_find_devs_with = util.find_devs_with
        try:
            util.find_devs_with = my_devs_with
            self.assertEqual(["/dev/sdb", "/dev/sr0", "/dev/vdb"],
                             ds.find_candidate_devs())
        finally:
            util.find_devs_with = orig_find_devs_with


@mock.patch(DS_PATH + '.net.get_interfaces_by_mac', mock.Mock(return_value={}))
class TestOpenNebulaNetwork(unittest.TestCase):

    system_nics = ('eth0', 'ens3')

    def test_context_devname(self):
        """Verify context_devname correctly returns mac and name."""
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH1_MAC': '02:00:0a:12:0f:0f', }
        expected = {
            '02:00:0a:12:01:01': 'ETH0',
            '02:00:0a:12:0f:0f': 'ETH1', }
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        self.assertEqual(expected, net.context_devname)

    def test_get_nameservers(self):
        """
        Verify get_nameservers('device') correctly returns DNS server addresses
        and search domains.
        """
        context = {
            'DNS': '1.2.3.8',
            'ETH0_DNS': '1.2.3.6 1.2.3.7',
            'ETH0_SEARCH_DOMAIN': 'example.com example.org', }
        expected = {
            'addresses': ['1.2.3.6', '1.2.3.7', '1.2.3.8'],
            'search': ['example.com', 'example.org']}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_nameservers('eth0')
        self.assertEqual(expected, val)

    def test_get_mtu(self):
        """Verify get_mtu('device') correctly returns MTU size."""
        context = {'ETH0_MTU': '1280'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_mtu('eth0')
        self.assertEqual('1280', val)

    def test_get_ip(self):
        """Verify get_ip('device') correctly returns IPv4 address."""
        context = {'ETH0_IP': PUBLIC_IP}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip('eth0', MACADDR)
        self.assertEqual(PUBLIC_IP, val)

    def test_get_ip_emptystring(self):
        """
        Verify get_ip('device') correctly returns IPv4 address.
        It returns IP address created by MAC address if ETH0_IP has empty
        string.
        """
        context = {'ETH0_IP': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip('eth0', MACADDR)
        self.assertEqual(IP_BY_MACADDR, val)

    def test_get_ip6(self):
        """
        Verify get_ip6('device') correctly returns IPv6 address.
        In this case, IPv6 address is Given by ETH0_IP6.
        """
        context = {
            'ETH0_IP6': IP6_GLOBAL,
            'ETH0_IP6_ULA': '', }
        expected = [IP6_GLOBAL]
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip6('eth0')
        self.assertEqual(expected, val)

    def test_get_ip6_ula(self):
        """
        Verify get_ip6('device') correctly returns IPv6 address.
        In this case, IPv6 address is Given by ETH0_IP6_ULA.
        """
        context = {
            'ETH0_IP6': '',
            'ETH0_IP6_ULA': IP6_ULA, }
        expected = [IP6_ULA]
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip6('eth0')
        self.assertEqual(expected, val)

    def test_get_ip6_dual(self):
        """
        Verify get_ip6('device') correctly returns IPv6 address.
        In this case, IPv6 addresses are Given by ETH0_IP6 and ETH0_IP6_ULA.
        """
        context = {
            'ETH0_IP6': IP6_GLOBAL,
            'ETH0_IP6_ULA': IP6_ULA, }
        expected = [IP6_GLOBAL, IP6_ULA]
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip6('eth0')
        self.assertEqual(expected, val)

    def test_get_ip6_prefix(self):
        """
        Verify get_ip6_prefix('device') correctly returns IPv6 prefix.
        """
        context = {'ETH0_IP6_PREFIX_LENGTH': IP6_PREFIX}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip6_prefix('eth0')
        self.assertEqual(IP6_PREFIX, val)

    def test_get_ip6_prefix_emptystring(self):
        """
        Verify get_ip6_prefix('device') correctly returns IPv6 prefix.
        It returns default value '64' if ETH0_IP6_PREFIX_LENGTH has empty
        string.
        """
        context = {'ETH0_IP6_PREFIX_LENGTH': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_ip6_prefix('eth0')
        self.assertEqual('64', val)

    def test_get_gateway(self):
        """
        Verify get_gateway('device') correctly returns IPv4 default gateway
        address.
        """
        context = {'ETH0_GATEWAY': '1.2.3.5'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_gateway('eth0')
        self.assertEqual('1.2.3.5', val)

    def test_get_gateway6(self):
        """
        Verify get_gateway6('device') correctly returns IPv6 default gateway
        address.
        """
        context = {'ETH0_GATEWAY6': IP6_GW}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_gateway6('eth0')
        self.assertEqual(IP6_GW, val)

    def test_get_mask(self):
        """
        Verify get_mask('device') correctly returns IPv4 subnet mask.
        """
        context = {'ETH0_MASK': '255.255.0.0'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_mask('eth0')
        self.assertEqual('255.255.0.0', val)

    def test_get_mask_emptystring(self):
        """
        Verify get_mask('device') correctly returns IPv4 subnet mask.
        It returns default value '255.255.255.0' if ETH0_MASK has empty string.
        """
        context = {'ETH0_MASK': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_mask('eth0')
        self.assertEqual('255.255.255.0', val)

    def test_get_network(self):
        """
        Verify get_network('device') correctly returns IPv4 network address.
        """
        context = {'ETH0_NETWORK': '1.2.3.0'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_network('eth0', MACADDR)
        self.assertEqual('1.2.3.0', val)

    def test_get_network_emptystring(self):
        """
        Verify get_network('device') correctly returns IPv4 network address.
        It returns network address created by MAC address if ETH0_NETWORK has
        empty string.
        """
        context = {'ETH0_NETWORK': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_network('eth0', MACADDR)
        self.assertEqual('10.18.1.0', val)

    def test_get_field(self):
        """
        Verify get_field('device', 'name') returns *context* value.
        """
        context = {'ETH9_DUMMY': 'DUMMY_VALUE'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_field('eth9', 'dummy')
        self.assertEqual('DUMMY_VALUE', val)

    def test_get_field_withdefaultvalue(self):
        """
        Verify get_field('device', 'name', 'default value') returns *context*
        value.
        """
        context = {'ETH9_DUMMY': 'DUMMY_VALUE'}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_field('eth9', 'dummy', 'DEFAULT_VALUE')
        self.assertEqual('DUMMY_VALUE', val)

    def test_get_field_withdefaultvalue_emptycontext(self):
        """
        Verify get_field('device', 'name', 'default value') returns *default*
        value if context value is empty string.
        """
        context = {'ETH9_DUMMY': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_field('eth9', 'dummy', 'DEFAULT_VALUE')
        self.assertEqual('DEFAULT_VALUE', val)

    def test_get_field_emptycontext(self):
        """
        Verify get_field('device', 'name') returns None if context value is
        empty string.
        """
        context = {'ETH9_DUMMY': ''}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_field('eth9', 'dummy')
        self.assertEqual(None, val)

    def test_get_field_nonecontext(self):
        """
        Verify get_field('device', 'name') returns None if context value is
        None.
        """
        context = {'ETH9_DUMMY': None}
        net = ds.OpenNebulaNetwork(context, mock.Mock())
        val = net.get_field('eth9', 'dummy')
        self.assertEqual(None, val)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_gen_conf_gateway(self, m_get_phys_by_mac):
        """Test rendering with/without IPv4 gateway"""
        self.maxDiff = None
        # empty ETH0_GATEWAY
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_GATEWAY': '', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

        # set ETH0_GATEWAY
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_GATEWAY': '1.2.3.5', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'gateway4': '1.2.3.5',
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_gen_conf_gateway6(self, m_get_phys_by_mac):
        """Test rendering with/without IPv6 gateway"""
        self.maxDiff = None
        # empty ETH0_GATEWAY6
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_GATEWAY6': '', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

        # set ETH0_GATEWAY6
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_GATEWAY6': IP6_GW, }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'gateway6': IP6_GW,
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_gen_conf_ipv6address(self, m_get_phys_by_mac):
        """Test rendering with/without IPv6 address"""
        self.maxDiff = None
        # empty ETH0_IP6, ETH0_IP6_ULA, ETH0_IP6_PREFIX_LENGTH
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_IP6': '',
            'ETH0_IP6_ULA': '',
            'ETH0_IP6_PREFIX_LENGTH': '', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

        # set ETH0_IP6, ETH0_IP6_ULA, ETH0_IP6_PREFIX_LENGTH
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_IP6': IP6_GLOBAL,
            'ETH0_IP6_PREFIX_LENGTH': IP6_PREFIX,
            'ETH0_IP6_ULA': IP6_ULA, }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [
                            IP_BY_MACADDR + '/' + IP4_PREFIX,
                            IP6_GLOBAL + '/' + IP6_PREFIX,
                            IP6_ULA + '/' + IP6_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_gen_conf_dns(self, m_get_phys_by_mac):
        """Test rendering with/without DNS server, search domain"""
        self.maxDiff = None
        # empty DNS, ETH0_DNS, ETH0_SEARCH_DOMAIN
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'DNS': '',
            'ETH0_DNS': '',
            'ETH0_SEARCH_DOMAIN': '', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

        # set DNS, ETH0_DNS, ETH0_SEARCH_DOMAIN
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'DNS': '1.2.3.8',
            'ETH0_DNS': '1.2.3.6 1.2.3.7',
            'ETH0_SEARCH_DOMAIN': 'example.com example.org', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'nameservers': {
                            'addresses': ['1.2.3.6', '1.2.3.7', '1.2.3.8'],
                            'search': ['example.com', 'example.org']},
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_gen_conf_mtu(self, m_get_phys_by_mac):
        """Test rendering with/without MTU"""
        self.maxDiff = None
        # empty ETH0_MTU
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_MTU': '', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

        # set ETH0_MTU
        context = {
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_MTU': '1280', }
        for nic in self.system_nics:
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'mtu': '1280',
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork(context, mock.Mock())
            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_eth0(self, m_get_phys_by_mac):
        for nic in self.system_nics:
            m_get_phys_by_mac.return_value = {MACADDR: nic}
            net = ds.OpenNebulaNetwork({}, mock.Mock())
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/' + IP4_PREFIX]}}}

            self.assertEqual(net.gen_conf(), expected)

    @mock.patch(DS_PATH + ".get_physical_nics_by_mac")
    def test_distro_passed_through(self, m_get_physical_nics_by_mac):
        ds.OpenNebulaNetwork({}, mock.sentinel.distro)
        self.assertEqual(
            [mock.call(mock.sentinel.distro)],
            m_get_physical_nics_by_mac.call_args_list,
        )

    def test_eth0_override(self):
        self.maxDiff = None
        context = {
            'DNS': '1.2.3.8',
            'ETH0_DNS': '1.2.3.6 1.2.3.7',
            'ETH0_GATEWAY': '1.2.3.5',
            'ETH0_GATEWAY6': '',
            'ETH0_IP': IP_BY_MACADDR,
            'ETH0_IP6': '',
            'ETH0_IP6_PREFIX_LENGTH': '',
            'ETH0_IP6_ULA': '',
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_MASK': '255.255.0.0',
            'ETH0_MTU': '',
            'ETH0_NETWORK': '10.18.0.0',
            'ETH0_SEARCH_DOMAIN': '',
        }
        for nic in self.system_nics:
            net = ds.OpenNebulaNetwork(context, mock.Mock(),
                                       system_nics_by_mac={MACADDR: nic})
            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [IP_BY_MACADDR + '/16'],
                        'gateway4': '1.2.3.5',
                        'nameservers': {
                            'addresses': ['1.2.3.6', '1.2.3.7', '1.2.3.8']}}}}

            self.assertEqual(expected, net.gen_conf())

    def test_eth0_v4v6_override(self):
        self.maxDiff = None
        context = {
            'DNS': '1.2.3.8',
            'ETH0_DNS': '1.2.3.6 1.2.3.7',
            'ETH0_GATEWAY': '1.2.3.5',
            'ETH0_GATEWAY6': IP6_GW,
            'ETH0_IP': IP_BY_MACADDR,
            'ETH0_IP6': IP6_GLOBAL,
            'ETH0_IP6_PREFIX_LENGTH': IP6_PREFIX,
            'ETH0_IP6_ULA': IP6_ULA,
            'ETH0_MAC': '02:00:0a:12:01:01',
            'ETH0_MASK': '255.255.0.0',
            'ETH0_MTU': '1280',
            'ETH0_NETWORK': '10.18.0.0',
            'ETH0_SEARCH_DOMAIN': 'example.com example.org',
        }
        for nic in self.system_nics:
            net = ds.OpenNebulaNetwork(context, mock.Mock(),
                                       system_nics_by_mac={MACADDR: nic})

            expected = {
                'version': 2,
                'ethernets': {
                    nic: {
                        'match': {'macaddress': MACADDR},
                        'addresses': [
                            IP_BY_MACADDR + '/16',
                            IP6_GLOBAL + '/' + IP6_PREFIX,
                            IP6_ULA + '/' + IP6_PREFIX],
                        'gateway4': '1.2.3.5',
                        'gateway6': IP6_GW,
                        'nameservers': {
                            'addresses': ['1.2.3.6', '1.2.3.7', '1.2.3.8'],
                            'search': ['example.com', 'example.org']},
                        'mtu': '1280'}}}

            self.assertEqual(expected, net.gen_conf())

    def test_multiple_nics(self):
        """Test rendering multiple nics with names that differ from context."""
        self.maxDiff = None
        MAC_1 = "02:00:0a:12:01:01"
        MAC_2 = "02:00:0a:12:01:02"
        context = {
            'DNS': '1.2.3.8',
            'ETH0_DNS': '1.2.3.6 1.2.3.7',
            'ETH0_GATEWAY': '1.2.3.5',
            'ETH0_GATEWAY6': IP6_GW,
            'ETH0_IP': '10.18.1.1',
            'ETH0_IP6': IP6_GLOBAL,
            'ETH0_IP6_PREFIX_LENGTH': '',
            'ETH0_IP6_ULA': IP6_ULA,
            'ETH0_MAC': MAC_2,
            'ETH0_MASK': '255.255.0.0',
            'ETH0_MTU': '1280',
            'ETH0_NETWORK': '10.18.0.0',
            'ETH0_SEARCH_DOMAIN': 'example.com',
            'ETH3_DNS': '10.3.1.2',
            'ETH3_GATEWAY': '10.3.0.1',
            'ETH3_GATEWAY6': '',
            'ETH3_IP': '10.3.1.3',
            'ETH3_IP6': '',
            'ETH3_IP6_PREFIX_LENGTH': '',
            'ETH3_IP6_ULA': '',
            'ETH3_MAC': MAC_1,
            'ETH3_MASK': '255.255.0.0',
            'ETH3_MTU': '',
            'ETH3_NETWORK': '10.3.0.0',
            'ETH3_SEARCH_DOMAIN': 'third.example.com third.example.org',
        }
        net = ds.OpenNebulaNetwork(
            context,
            mock.Mock(),
            system_nics_by_mac={MAC_1: 'enp0s25', MAC_2: 'enp1s2'}
        )

        expected = {
            'version': 2,
            'ethernets': {
                'enp1s2': {
                    'match': {'macaddress': MAC_2},
                    'addresses': [
                        '10.18.1.1/16',
                        IP6_GLOBAL + '/64',
                        IP6_ULA + '/64'],
                    'gateway4': '1.2.3.5',
                    'gateway6': IP6_GW,
                    'nameservers': {
                        'addresses': ['1.2.3.6', '1.2.3.7', '1.2.3.8'],
                        'search': ['example.com']},
                    'mtu': '1280'},
                'enp0s25': {
                    'match': {'macaddress': MAC_1},
                    'addresses': ['10.3.1.3/16'],
                    'gateway4': '10.3.0.1',
                    'nameservers': {
                        'addresses': ['10.3.1.2', '1.2.3.8'],
                        'search': [
                            'third.example.com',
                            'third.example.org']}}}}

        self.assertEqual(expected, net.gen_conf())


class TestParseShellConfig:
    @pytest.mark.allow_subp_for("bash")
    def test_no_seconds(self):
        cfg = '\n'.join(["foo=bar", "SECONDS=2", "xx=foo"])
        # we could test 'sleep 2', but that would make the test run slower.
        ret = ds.parse_shell_config(cfg)
        assert ret == {"foo": "bar", "xx": "foo"}


class TestGetPhysicalNicsByMac:
    @pytest.mark.parametrize(
        "interfaces_by_mac,physical_devs,expected_return",
        [
            # No interfaces => empty return
            ({}, [], {}),
            # Only virtual interface => empty return
            ({"mac1": "virtual0"}, [], {}),
            # Only physical interface => it is returned
            ({"mac2": "physical0"}, ["physical0"], {"mac2": "physical0"}),
            # Combination of physical and virtual => only physical returned
            (
                {"mac3": "physical1", "mac4": "virtual1"},
                ["physical1"],
                {"mac3": "physical1"},
            ),
        ],
    )
    def test(self, interfaces_by_mac, physical_devs, expected_return):
        distro = mock.Mock()
        distro.networking.is_physical.side_effect = (
            lambda devname: devname in physical_devs
        )
        with mock.patch(
            DS_PATH + ".net.get_interfaces_by_mac",
            return_value=interfaces_by_mac,
        ):
            assert expected_return == ds.get_physical_nics_by_mac(distro)


def populate_context_dir(path, variables):
    data = "# Context variables generated by OpenNebula\n"
    for k, v in variables.items():
        data += ("%s='%s'\n" % (k.upper(), v.replace(r"'", r"'\''")))
    populate_dir(path, {'context.sh': data})

# vi: ts=4 expandtab



# This file is part of cloud-init. See LICENSE file for license information.

"""Tests related to cloudinit.stages module."""

import os
import stat

import pytest

from cloudinit import stages
from cloudinit import sources
from cloudinit.sources import NetworkConfigSource

from cloudinit.event import EventType
from cloudinit.util import write_file

from cloudinit.tests.helpers import CiTestCase, mock

TEST_INSTANCE_ID = 'i-testing'


class FakeDataSource(sources.DataSource):

    def __init__(self, paths=None, userdata=None, vendordata=None,
                 network_config=''):
        super(FakeDataSource, self).__init__({}, None, paths=paths)
        self.metadata = {'instance-id': TEST_INSTANCE_ID}
        self.userdata_raw = userdata
        self.vendordata_raw = vendordata
        self._network_config = None
        if network_config:   # Permit for None value to setup attribute
            self._network_config = network_config

    @property
    def network_config(self):
        return self._network_config

    def _get_data(self):
        return True


class TestInit(CiTestCase):
    with_logs = True
    allowed_subp = False

    def setUp(self):
        super(TestInit, self).setUp()
        self.tmpdir = self.tmp_dir()
        self.init = stages.Init()
        # Setup fake Paths for Init to reference
        self.init._cfg = {'system_info': {
            'distro': 'ubuntu', 'paths': {'cloud_dir': self.tmpdir,
                                          'run_dir': self.tmpdir}}}
        self.init.datasource = FakeDataSource(paths=self.init.paths)

    def test_wb__find_networking_config_disabled(self):
        """find_networking_config returns no config when disabled."""
        disable_file = os.path.join(
            self.init.paths.get_cpath('data'), 'upgraded-network')
        write_file(disable_file, '')
        self.assertEqual(
            (None, disable_file),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_disabled_by_kernel(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns when disabled by kernel cmdline."""
        m_cmdline.return_value = {'config': 'disabled'}
        m_initramfs.return_value = {'config': ['fake_initrd']}
        self.assertEqual(
            (None, NetworkConfigSource.cmdline),
            self.init._find_networking_config())
        self.assertEqual('DEBUG: network config disabled by cmdline\n',
                         self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_disabled_by_initrd(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns when disabled by kernel cmdline."""
        m_cmdline.return_value = {}
        m_initramfs.return_value = {'config': 'disabled'}
        self.assertEqual(
            (None, NetworkConfigSource.initramfs),
            self.init._find_networking_config())
        self.assertEqual('DEBUG: network config disabled by initramfs\n',
                         self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_disabled_by_datasrc(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns when disabled by datasource cfg."""
        m_cmdline.return_value = {}  # Kernel doesn't disable networking
        m_initramfs.return_value = {}  # initramfs doesn't disable networking
        self.init._cfg = {'system_info': {'paths': {'cloud_dir': self.tmpdir}},
                          'network': {}}  # system config doesn't disable

        self.init.datasource = FakeDataSource(
            network_config={'config': 'disabled'})
        self.assertEqual(
            (None, NetworkConfigSource.ds),
            self.init._find_networking_config())
        self.assertEqual('DEBUG: network config disabled by ds\n',
                         self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_disabled_by_sysconfig(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns when disabled by system config."""
        m_cmdline.return_value = {}  # Kernel doesn't disable networking
        m_initramfs.return_value = {}  # initramfs doesn't disable networking
        self.init._cfg = {'system_info': {'paths': {'cloud_dir': self.tmpdir}},
                          'network': {'config': 'disabled'}}
        self.assertEqual(
            (None, NetworkConfigSource.system_cfg),
            self.init._find_networking_config())
        self.assertEqual('DEBUG: network config disabled by system_cfg\n',
                         self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test__find_networking_config_uses_datasrc_order(
            self, m_cmdline, m_initramfs):
        """find_networking_config should check sources in DS defined order"""
        # cmdline and initramfs, which would normally be preferred over other
        # sources, disable networking; in this case, though, the DS moves them
        # later so its own config is preferred
        m_cmdline.return_value = {'config': 'disabled'}
        m_initramfs.return_value = {'config': 'disabled'}

        ds_net_cfg = {'config': {'needle': True}}
        self.init.datasource = FakeDataSource(network_config=ds_net_cfg)
        self.init.datasource.network_config_sources = [
            NetworkConfigSource.ds, NetworkConfigSource.system_cfg,
            NetworkConfigSource.cmdline, NetworkConfigSource.initramfs]

        self.assertEqual(
            (ds_net_cfg, NetworkConfigSource.ds),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test__find_networking_config_warns_if_datasrc_uses_invalid_src(
            self, m_cmdline, m_initramfs):
        """find_networking_config should check sources in DS defined order"""
        ds_net_cfg = {'config': {'needle': True}}
        self.init.datasource = FakeDataSource(network_config=ds_net_cfg)
        self.init.datasource.network_config_sources = [
            'invalid_src', NetworkConfigSource.ds]

        self.assertEqual(
            (ds_net_cfg, NetworkConfigSource.ds),
            self.init._find_networking_config())
        self.assertIn('WARNING: data source specifies an invalid network'
                      ' cfg_source: invalid_src',
                      self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test__find_networking_config_warns_if_datasrc_uses_unavailable_src(
            self, m_cmdline, m_initramfs):
        """find_networking_config should check sources in DS defined order"""
        ds_net_cfg = {'config': {'needle': True}}
        self.init.datasource = FakeDataSource(network_config=ds_net_cfg)
        self.init.datasource.network_config_sources = [
            NetworkConfigSource.fallback, NetworkConfigSource.ds]

        self.assertEqual(
            (ds_net_cfg, NetworkConfigSource.ds),
            self.init._find_networking_config())
        self.assertIn('WARNING: data source specifies an unavailable network'
                      ' cfg_source: fallback',
                      self.logs.getvalue())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_returns_kernel(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns kernel cmdline config if present."""
        expected_cfg = {'config': ['fakekernel']}
        m_cmdline.return_value = expected_cfg
        m_initramfs.return_value = {'config': ['fake_initrd']}
        self.init._cfg = {'system_info': {'paths': {'cloud_dir': self.tmpdir}},
                          'network': {'config': ['fakesys_config']}}
        self.init.datasource = FakeDataSource(
            network_config={'config': ['fakedatasource']})
        self.assertEqual(
            (expected_cfg, NetworkConfigSource.cmdline),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_returns_initramfs(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns kernel cmdline config if present."""
        expected_cfg = {'config': ['fake_initrd']}
        m_cmdline.return_value = {}
        m_initramfs.return_value = expected_cfg
        self.init._cfg = {'system_info': {'paths': {'cloud_dir': self.tmpdir}},
                          'network': {'config': ['fakesys_config']}}
        self.init.datasource = FakeDataSource(
            network_config={'config': ['fakedatasource']})
        self.assertEqual(
            (expected_cfg, NetworkConfigSource.initramfs),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_returns_system_cfg(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns system config when present."""
        m_cmdline.return_value = {}  # No kernel network config
        m_initramfs.return_value = {}  # no initramfs network config
        expected_cfg = {'config': ['fakesys_config']}
        self.init._cfg = {'system_info': {'paths': {'cloud_dir': self.tmpdir}},
                          'network': expected_cfg}
        self.init.datasource = FakeDataSource(
            network_config={'config': ['fakedatasource']})
        self.assertEqual(
            (expected_cfg, NetworkConfigSource.system_cfg),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_returns_datasrc_cfg(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns datasource net config if present."""
        m_cmdline.return_value = {}  # No kernel network config
        m_initramfs.return_value = {}  # no initramfs network config
        # No system config for network in setUp
        expected_cfg = {'config': ['fakedatasource']}
        self.init.datasource = FakeDataSource(network_config=expected_cfg)
        self.assertEqual(
            (expected_cfg, NetworkConfigSource.ds),
            self.init._find_networking_config())

    @mock.patch('cloudinit.stages.cmdline.read_initramfs_config')
    @mock.patch('cloudinit.stages.cmdline.read_kernel_cmdline_config')
    def test_wb__find_networking_config_returns_fallback(
            self, m_cmdline, m_initramfs):
        """find_networking_config returns fallback config if not defined."""
        m_cmdline.return_value = {}  # Kernel doesn't disable networking
        m_initramfs.return_value = {}  # no initramfs network config
        # Neither datasource nor system_info disable or provide network

        fake_cfg = {'config': [{'type': 'physical', 'name': 'eth9'}],
                    'version': 1}

        def fake_generate_fallback():
            return fake_cfg

        # Monkey patch distro which gets cached on self.init
        distro = self.init.distro
        distro.generate_fallback_config = fake_generate_fallback
        self.assertEqual(
            (fake_cfg, NetworkConfigSource.fallback),
            self.init._find_networking_config())
        self.assertNotIn('network config disabled', self.logs.getvalue())

    def test_apply_network_config_disabled(self):
        """Log when network is disabled by upgraded-network."""
        disable_file = os.path.join(
            self.init.paths.get_cpath('data'), 'upgraded-network')

        def fake_network_config():
            return (None, disable_file)

        self.init._find_networking_config = fake_network_config

        self.init.apply_network_config(True)
        self.assertIn(
            'INFO: network config is disabled by %s' % disable_file,
            self.logs.getvalue())

    @mock.patch('cloudinit.net.get_interfaces_by_mac')
    @mock.patch('cloudinit.distros.ubuntu.Distro')
    def test_apply_network_on_new_instance(self, m_ubuntu, m_macs):
        """Call distro apply_network_config methods on is_new_instance."""
        net_cfg = {
            'version': 1, 'config': [
                {'subnets': [{'type': 'dhcp'}], 'type': 'physical',
                 'name': 'eth9', 'mac_address': '42:42:42:42:42:42'}]}

        def fake_network_config():
            return net_cfg, NetworkConfigSource.fallback

        m_macs.return_value = {'42:42:42:42:42:42': 'eth9'}

        self.init._find_networking_config = fake_network_config
        self.init.apply_network_config(True)
        self.init.distro.apply_network_config_names.assert_called_with(net_cfg)
        self.init.distro.apply_network_config.assert_called_with(
            net_cfg, bring_up=True)

    @mock.patch('cloudinit.distros.ubuntu.Distro')
    def test_apply_network_on_same_instance_id(self, m_ubuntu):
        """Only call distro.apply_network_config_names on same instance id."""
        old_instance_id = os.path.join(
            self.init.paths.get_cpath('data'), 'instance-id')
        write_file(old_instance_id, TEST_INSTANCE_ID)
        net_cfg = {
            'version': 1, 'config': [
                {'subnets': [{'type': 'dhcp'}], 'type': 'physical',
                 'name': 'eth9', 'mac_address': '42:42:42:42:42:42'}]}

        def fake_network_config():
            return net_cfg, NetworkConfigSource.fallback

        self.init._find_networking_config = fake_network_config
        self.init.apply_network_config(True)
        self.init.distro.apply_network_config_names.assert_called_with(net_cfg)
        self.init.distro.apply_network_config.assert_not_called()
        self.assertIn(
            'No network config applied. Neither a new instance'
            " nor datasource network update on '%s' event" % EventType.BOOT,
            self.logs.getvalue())

    @mock.patch('cloudinit.net.get_interfaces_by_mac')
    @mock.patch('cloudinit.distros.ubuntu.Distro')
    def test_apply_network_on_datasource_allowed_event(self, m_ubuntu, m_macs):
        """Apply network if datasource.update_metadata permits BOOT event."""
        old_instance_id = os.path.join(
            self.init.paths.get_cpath('data'), 'instance-id')
        write_file(old_instance_id, TEST_INSTANCE_ID)
        net_cfg = {
            'version': 1, 'config': [
                {'subnets': [{'type': 'dhcp'}], 'type': 'physical',
                 'name': 'eth9', 'mac_address': '42:42:42:42:42:42'}]}

        def fake_network_config():
            return net_cfg, NetworkConfigSource.fallback

        m_macs.return_value = {'42:42:42:42:42:42': 'eth9'}

        self.init._find_networking_config = fake_network_config
        self.init.datasource = FakeDataSource(paths=self.init.paths)
        self.init.datasource.update_events = {'network': [EventType.BOOT]}
        self.init.apply_network_config(True)
        self.init.distro.apply_network_config_names.assert_called_with(net_cfg)
        self.init.distro.apply_network_config.assert_called_with(
            net_cfg, bring_up=True)


class TestInit_InitializeFilesystem:
    """Tests for cloudinit.stages.Init._initialize_filesystem.

    TODO: Expand these tests to cover all of _initialize_filesystem's behavior.
    """

    @pytest.yield_fixture
    def init(self, paths):
        """A fixture which yields a stages.Init instance with paths and cfg set

        As it is replaced with a mock, consumers of this fixture can set
        `init.cfg` if the default empty dict configuration is not appropriate.
        """
        with mock.patch(
            "cloudinit.stages.Init.cfg", mock.PropertyMock(return_value={})
        ):
            with mock.patch("cloudinit.stages.util.ensure_dirs"):
                init = stages.Init()
                init._paths = paths
                yield init

    @mock.patch("cloudinit.stages.util.ensure_file")
    def test_ensure_file_not_called_if_no_log_file_configured(
        self, m_ensure_file, init
    ):
        """If no log file is configured, we should not ensure its existence."""
        init.cfg = {}

        init._initialize_filesystem()

        assert 0 == m_ensure_file.call_count

    def test_log_files_existence_is_ensured_if_configured(self, init, tmpdir):
        """If a log file is configured, we should ensure its existence."""
        log_file = tmpdir.join("cloud-init.log")
        init.cfg = {"def_log_file": str(log_file)}

        init._initialize_filesystem()

        assert log_file.exists

    def test_existing_file_permissions_are_not_modified(self, init, tmpdir):
        """If the log file already exists, we should not modify its permissions

        See https://bugs.launchpad.net/cloud-init/+bug/1900837.
        """
        # Use a mode that will never be made the default so this test will
        # always be valid
        mode = 0o606
        log_file = tmpdir.join("cloud-init.log")
        log_file.ensure()
        log_file.chmod(mode)
        init.cfg = {"def_log_file": str(log_file)}

        init._initialize_filesystem()

        assert mode == stat.S_IMODE(log_file.stat().mode)

# vi: ts=4 expandtab



"""Integration test for the runcmd module.

This test specifies a command to be executed by the ``runcmd`` module
and then checks if that command was executed during boot.

(This is ported from
``tests/cloud_tests/testcases/modules/runcmd.yaml``.)"""

import pytest


USER_DATA = """\
#cloud-config
runcmd:
 - echo cloud-init run cmd test > /var/tmp/run_cmd
"""


@pytest.mark.ci
class TestRuncmd:

    @pytest.mark.user_data(USER_DATA)
    def test_runcmd(self, client):
        runcmd_output = client.read_from_file("/var/tmp/run_cmd")
        assert runcmd_output.strip() == "cloud-init run cmd test"



"""Integration test of the cc_power_state_change module.

Test that the power state config options work as expected.
"""

import time

import pytest

from tests.integration_tests.clouds import IntegrationCloud
from tests.integration_tests.instances import IntegrationInstance
from tests.integration_tests.log_utils import verify_ordered_items_in_text

USER_DATA = """\
#cloud-config
power_state:
  delay: {delay}
  mode: {mode}
  message: msg
  timeout: {timeout}
  condition: {condition}
"""


def _detect_reboot(instance: IntegrationInstance):
    # We'll wait for instance up here, but we don't know if we're
    # detecting the first boot or second boot, so we also check
    # the logs to ensure we've booted twice. If the logs show we've
    # only booted once, wait until we've booted twice
    instance.instance.wait()
    for _ in range(600):
        try:
            log = instance.read_from_file('/var/log/cloud-init.log')
            boot_count = log.count("running 'init-local'")
            if boot_count == 1:
                instance.instance.wait()
            elif boot_count > 1:
                break
        except Exception:
            pass
        time.sleep(1)
    else:
        raise Exception('Could not detect reboot')


def _can_connect(instance):
    return instance.execute('true').ok


# This test is marked unstable because even though it should be able to
# run anywhere, I can only get it to run in an lxd container, and even then
# occasionally some timing issues will crop up.
@pytest.mark.unstable
@pytest.mark.sru_2020_11
@pytest.mark.ubuntu
@pytest.mark.lxd_container
class TestPowerChange:
    @pytest.mark.parametrize('mode,delay,timeout,expected', [
        ('poweroff', 'now', '10', 'will execute: shutdown -P now msg'),
        ('reboot', 'now', '0', 'will execute: shutdown -r now msg'),
        ('halt', '+1', '0', 'will execute: shutdown -H +1 msg'),
    ])
    def test_poweroff(self, session_cloud: IntegrationCloud,
                      mode, delay, timeout, expected):
        with session_cloud.launch(
            user_data=USER_DATA.format(
                delay=delay, mode=mode, timeout=timeout, condition='true'),
            launch_kwargs={'wait': False},
        ) as instance:
            if mode == 'reboot':
                _detect_reboot(instance)
            else:
                instance.instance.wait_for_stop()
                instance.instance.start(wait=True)
            log = instance.read_from_file('/var/log/cloud-init.log')
            assert _can_connect(instance)
        lines_to_check = [
            'Running module power-state-change',
            expected,
            "running 'init-local'",
            'config-power-state-change already ran',
        ]
        verify_ordered_items_in_text(lines_to_check, log)

    @pytest.mark.user_data(USER_DATA.format(delay='0', mode='poweroff',
                                            timeout='0', condition='false'))
    def test_poweroff_false_condition(self, client: IntegrationInstance):
        log = client.read_from_file('/var/log/cloud-init.log')
        assert _can_connect(client)
        assert 'Condition was false. Will not perform state change' in log



# This file is part of cloud-init. See LICENSE file for license information.

"""Tests for cloudinit.dhclient_hook."""

from cloudinit import dhclient_hook as dhc
from cloudinit.tests.helpers import CiTestCase, dir2dict, populate_dir

import argparse
import json
import os
from unittest import mock


class TestDhclientHook(CiTestCase):

    ex_env = {
        'interface': 'eth0',
        'new_dhcp_lease_time': '3600',
        'new_host_name': 'x1',
        'new_ip_address': '10.145.210.163',
        'new_subnet_mask': '255.255.255.0',
        'old_host_name': 'x1',
        'PATH': '/usr/sbin:/usr/bin:/sbin:/bin',
        'pid': '614',
        'reason': 'BOUND',
    }

    # some older versions of dhclient put the same content,
    # but in upper case with DHCP4_ instead of new_
    ex_env_dhcp4 = {
        'REASON': 'BOUND',
        'DHCP4_dhcp_lease_time': '3600',
        'DHCP4_host_name': 'x1',
        'DHCP4_ip_address': '10.145.210.163',
        'DHCP4_subnet_mask': '255.255.255.0',
        'INTERFACE': 'eth0',
        'PATH': '/usr/sbin:/usr/bin:/sbin:/bin',
        'pid': '614',
    }

    expected = {
        'dhcp_lease_time': '3600',
        'host_name': 'x1',
        'ip_address': '10.145.210.163',
        'subnet_mask': '255.255.255.0'}

    def setUp(self):
        super(TestDhclientHook, self).setUp()
        self.tmp = self.tmp_dir()

    def test_handle_args(self):
        """quick test of call to handle_args."""
        nic = 'eth0'
        args = argparse.Namespace(event=dhc.UP, interface=nic)
        with mock.patch.dict("os.environ", clear=True, values=self.ex_env):
            dhc.handle_args(dhc.NAME, args, data_d=self.tmp)
        found = dir2dict(self.tmp + os.path.sep)
        self.assertEqual([nic + ".json"], list(found.keys()))
        self.assertEqual(self.expected, json.loads(found[nic + ".json"]))

    def test_run_hook_up_creates_dir(self):
        """If dir does not exist, run_hook should create it."""
        subd = self.tmp_path("subdir", self.tmp)
        nic = 'eth1'
        dhc.run_hook(nic, 'up', data_d=subd, env=self.ex_env)
        self.assertEqual(
            set([nic + ".json"]), set(dir2dict(subd + os.path.sep)))

    def test_run_hook_up(self):
        """Test expected use of run_hook_up."""
        nic = 'eth0'
        dhc.run_hook(nic, 'up', data_d=self.tmp, env=self.ex_env)
        found = dir2dict(self.tmp + os.path.sep)
        self.assertEqual([nic + ".json"], list(found.keys()))
        self.assertEqual(self.expected, json.loads(found[nic + ".json"]))

    def test_run_hook_up_dhcp4_prefix(self):
        """Test run_hook filters correctly with older DHCP4_ data."""
        nic = 'eth0'
        dhc.run_hook(nic, 'up', data_d=self.tmp, env=self.ex_env_dhcp4)
        found = dir2dict(self.tmp + os.path.sep)
        self.assertEqual([nic + ".json"], list(found.keys()))
        self.assertEqual(self.expected, json.loads(found[nic + ".json"]))

    def test_run_hook_down_deletes(self):
        """down should delete the created json file."""
        nic = 'eth1'
        populate_dir(
            self.tmp, {nic + ".json": "{'abcd'}", 'myfile.txt': 'text'})
        dhc.run_hook(nic, 'down', data_d=self.tmp, env={'old_host_name': 'x1'})
        self.assertEqual(
            set(['myfile.txt']),
            set(dir2dict(self.tmp + os.path.sep)))

    def test_get_parser(self):
        """Smoke test creation of get_parser."""
        # cloud-init main uses 'action'.
        event, interface = (dhc.UP, 'mynic0')
        self.assertEqual(
            argparse.Namespace(event=event, interface=interface,
                               action=(dhc.NAME, dhc.handle_args)),
            dhc.get_parser().parse_args([event, interface]))


# vi: ts=4 expandtab



from unittest import mock

import pytest

from cloudinit.config.cc_resolv_conf import generate_resolv_conf


EXPECTED_HEADER = """\
# Your system has been configured with 'manage-resolv-conf' set to true.
# As a result, cloud-init has written this file with configuration data
# that it has been provided. Cloud-init, by default, will write this file
# a single time (PER_ONCE).
#\n\n"""


class TestGenerateResolvConf:
    @mock.patch("cloudinit.config.cc_resolv_conf.templater.render_to_file")
    def test_default_target_fname_is_etc_resolvconf(self, m_render_to_file):
        generate_resolv_conf("templates/resolv.conf.tmpl", mock.MagicMock())

        assert [
            mock.call(mock.ANY, "/etc/resolv.conf", mock.ANY)
        ] == m_render_to_file.call_args_list

    @mock.patch("cloudinit.config.cc_resolv_conf.templater.render_to_file")
    def test_target_fname_is_used_if_passed(self, m_render_to_file):
        generate_resolv_conf(
            "templates/resolv.conf.tmpl", mock.MagicMock(), "/use/this/path"
        )

        assert [
            mock.call(mock.ANY, "/use/this/path", mock.ANY)
        ] == m_render_to_file.call_args_list

    # Patch in templater so we can assert on the actual generated content
    @mock.patch("cloudinit.templater.util.write_file")
    # Parameterise with the value to be passed to generate_resolv_conf as the
    # params parameter, and the expected line after the header as
    # expected_extra_line.
    @pytest.mark.parametrize(
        "params,expected_extra_line",
        [
            # No options
            ({}, None),
            # Just a true flag
            ({"options": {"foo": True}}, "options foo"),
            # Just a false flag
            ({"options": {"foo": False}}, None),
            # Just an option
            ({"options": {"foo": "some_value"}}, "options foo:some_value"),
            # A true flag and an option
            (
                {"options": {"foo": "some_value", "bar": True}},
                "options bar foo:some_value",
            ),
            # Two options
            (
                {"options": {"foo": "some_value", "bar": "other_value"}},
                "options bar:other_value foo:some_value",
            ),
            # Everything
            (
                {
                    "options": {
                        "foo": "some_value",
                        "bar": "other_value",
                        "baz": False,
                        "spam": True,
                    }
                },
                "options spam bar:other_value foo:some_value",
            ),
        ],
    )
    def test_flags_and_options(
        self, m_write_file, params, expected_extra_line
    ):
        generate_resolv_conf("templates/resolv.conf.tmpl", params)

        expected_content = EXPECTED_HEADER
        if expected_extra_line is not None:
            # If we have any extra lines, expect a trailing newline
            expected_content += "\n".join([expected_extra_line, ""])
        assert [
            mock.call(mock.ANY, expected_content, mode=mock.ANY)
        ] == m_write_file.call_args_list



# This file is part of cloud-init. See LICENSE file for license information.

"""Tests cc_apt_pipelining handler"""

import cloudinit.config.cc_apt_pipelining as cc_apt_pipelining

from cloudinit.tests.helpers import CiTestCase, mock


class TestAptPipelining(CiTestCase):

    @mock.patch('cloudinit.config.cc_apt_pipelining.util.write_file')
    def test_not_disabled_by_default(self, m_write_file):
        """ensure that default behaviour is to not disable pipelining"""
        cc_apt_pipelining.handle('foo', {}, None, mock.MagicMock(), None)
        self.assertEqual(0, m_write_file.call_count)

    @mock.patch('cloudinit.config.cc_apt_pipelining.util.write_file')
    def test_false_disables_pipelining(self, m_write_file):
        """ensure that pipelining can be disabled with correct config"""
        cc_apt_pipelining.handle(
            'foo', {'apt_pipelining': 'false'}, None, mock.MagicMock(), None)
        self.assertEqual(1, m_write_file.call_count)
        args, _ = m_write_file.call_args
        self.assertEqual(cc_apt_pipelining.DEFAULT_FILE, args[0])
        self.assertIn('Pipeline-Depth "0"', args[1])

# vi: ts=4 expandtab



"""Integration test for the set_password module.

This test specifies a combination of user/password pairs, and ensures that the
system has the correct passwords set.

There are two tests run here: one tests chpasswd's list being a YAML list, the
other tests chpasswd's list being a string.  Both expect the same results, so
they use a mixin to share their test definitions, because we can (of course)
only specify one user-data per instance.
"""
import crypt

import pytest
import yaml


COMMON_USER_DATA = """\
#cloud-config
ssh_pwauth: yes
users:
  - default
  - name: tom
    # md5 gotomgo
    passwd: "$1$S7$tT1BEDIYrczeryDQJfdPe0"
    lock_passwd: false
  - name: dick
    # md5 gocubsgo
    passwd: "$1$ssisyfpf$YqvuJLfrrW6Cg/l53Pi1n1"
    lock_passwd: false
  - name: harry
    # sha512 goharrygo
    passwd: "$6$LF$9Z2p6rWK6TNC1DC6393ec0As.18KRAvKDbfsGJEdWN3sRQRwpdfoh37EQ3y\
Uh69tP4GSrGW5XKHxMLiKowJgm/"
    lock_passwd: false
  - name: jane
    # sha256 gojanego
    passwd: "$5$iW$XsxmWCdpwIW8Yhv.Jn/R3uk6A4UaicfW5Xp7C9p9pg."
    lock_passwd: false
  - name: "mikey"
    lock_passwd: false
"""

LIST_USER_DATA = COMMON_USER_DATA + """
chpasswd:
  list:
    - tom:mypassword123!
    - dick:RANDOM
    - harry:RANDOM
    - mikey:$5$xZ$B2YGGEx2AOf4PeW48KC6.QyT1W2B4rZ9Qbltudtha89
"""

STRING_USER_DATA = COMMON_USER_DATA + """
chpasswd:
    list: |
      tom:mypassword123!
      dick:RANDOM
      harry:RANDOM
      mikey:$5$xZ$B2YGGEx2AOf4PeW48KC6.QyT1W2B4rZ9Qbltudtha89
"""

USERS_DICTS = yaml.safe_load(COMMON_USER_DATA)["users"]
USERS_PASSWD_VALUES = {
    user_dict["name"]: user_dict["passwd"]
    for user_dict in USERS_DICTS
    if "name" in user_dict and "passwd" in user_dict
}


class Mixin:
    """Shared test definitions."""

    def _fetch_and_parse_etc_shadow(self, class_client):
        """Fetch /etc/shadow and parse it into Python data structures

        Returns: ({user: password}, [duplicate, users])
        """
        shadow_content = class_client.read_from_file("/etc/shadow")
        users = {}
        dupes = []
        for line in shadow_content.splitlines():
            user, encpw = line.split(":")[0:2]
            if user in users:
                dupes.append(user)
            users[user] = encpw
        return users, dupes

    def test_no_duplicate_users_in_shadow(self, class_client):
        """Confirm that set_passwords has not added duplicate shadow entries"""
        _, dupes = self._fetch_and_parse_etc_shadow(class_client)

        assert [] == dupes

    def test_password_in_users_dict_set_correctly(self, class_client):
        """Test that the password specified in the users dict is set."""
        shadow_users, _ = self._fetch_and_parse_etc_shadow(class_client)
        assert USERS_PASSWD_VALUES["jane"] == shadow_users["jane"]

    def test_password_in_chpasswd_list_set_correctly(self, class_client):
        """Test that a chpasswd password overrides one in the users dict."""
        shadow_users, _ = self._fetch_and_parse_etc_shadow(class_client)
        mikey_hash = "$5$xZ$B2YGGEx2AOf4PeW48KC6.QyT1W2B4rZ9Qbltudtha89"
        assert mikey_hash == shadow_users["mikey"]

    def test_random_passwords_set_correctly(self, class_client):
        """Test that RANDOM chpasswd entries replace users dict passwords."""
        shadow_users, _ = self._fetch_and_parse_etc_shadow(class_client)

        # These should have been changed
        assert shadow_users["harry"] != USERS_PASSWD_VALUES["harry"]
        assert shadow_users["dick"] != USERS_PASSWD_VALUES["dick"]

        # To random passwords
        assert shadow_users["harry"].startswith("$")
        assert shadow_users["dick"].startswith("$")

        # Which are not the same
        assert shadow_users["harry"] != shadow_users["dick"]

    def test_explicit_password_set_correctly(self, class_client):
        """Test that an explicitly-specified password is set correctly."""
        shadow_users, _ = self._fetch_and_parse_etc_shadow(class_client)

        fmt_and_salt = shadow_users["tom"].rsplit("$", 1)[0]
        expected_value = crypt.crypt("mypassword123!", fmt_and_salt)

        assert expected_value == shadow_users["tom"]

    def test_shadow_expected_users(self, class_client):
        """Test that the right set of users is in /etc/shadow."""
        shadow = class_client.read_from_file("/etc/shadow")
        for user_dict in USERS_DICTS:
            if "name" in user_dict:
                assert "{}:".format(user_dict["name"]) in shadow

    def test_sshd_config(self, class_client):
        """Test that SSH password auth is enabled."""
        sshd_config = class_client.read_from_file("/etc/ssh/sshd_config")
        # We look for the exact line match, to avoid a commented line matching
        assert "PasswordAuthentication yes" in sshd_config.splitlines()


@pytest.mark.ci
@pytest.mark.user_data(LIST_USER_DATA)
class TestPasswordList(Mixin):
    """Launch an instance with LIST_USER_DATA, ensure Mixin tests pass."""


@pytest.mark.ci
@pytest.mark.user_data(STRING_USER_DATA)
class TestPasswordListString(Mixin):
    """Launch an instance with STRING_USER_DATA, ensure Mixin tests pass."""



"""Integration test for gh-632.

Verify that if cloud-init is using DataSourceRbxCloud, there is
no traceback if the metadata disk cannot be found.
"""

import pytest

from tests.integration_tests.instances import IntegrationInstance


# With some datasource hacking, we can run this on a NoCloud instance
@pytest.mark.lxd_container
@pytest.mark.lxd_vm
@pytest.mark.sru_2020_11
def test_datasource_rbx_no_stacktrace(client: IntegrationInstance):
    client.write_to_file(
        '/etc/cloud/cloud.cfg.d/90_dpkg.cfg',
        'datasource_list: [ RbxCloud, NoCloud ]\n',
    )
    client.write_to_file(
        '/etc/cloud/ds-identify.cfg',
        'policy: enabled\n',
    )
    client.execute('cloud-init clean --logs')
    client.restart()

    log = client.read_from_file('/var/log/cloud-init.log')
    assert 'WARNING' not in log
    assert 'Traceback' not in log
    assert 'Failed to load metadata and userdata' not in log
    assert ("Getting data from <class 'cloudinit.sources.DataSourceRbxCloud."
            "DataSourceRbxCloud'> failed") not in log



import json

from cloudinit import helpers
from cloudinit import distros
from cloudinit.sources import DataSourceRbxCloud as ds
from cloudinit.tests.helpers import mock, CiTestCase, populate_dir
from cloudinit import subp

DS_PATH = "cloudinit.sources.DataSourceRbxCloud"

CRYPTO_PASS = "$6$uktth46t$FvpDzFD2iL9YNZIG1Epz7957hJqbH0f" \
              "QKhnzcfBcUhEodGAWRqTy7tYG4nEW7SUOYBjxOSFIQW5" \
              "tToyGP41.s1"

CLOUD_METADATA = {
    "vm": {
        "memory": 4,
        "cpu": 2,
        "name": "vm-image-builder",
        "_id": "5beab44f680cffd11f0e60fc"
    },
    "additionalMetadata": {
        "username": "guru",
        "sshKeys": ["ssh-rsa ..."],
        "password": {
            "sha512": CRYPTO_PASS
        }
    },
    "disk": [
        {"size": 10, "type": "ssd",
         "name": "vm-image-builder-os",
         "_id": "5beab450680cffd11f0e60fe"},
        {"size": 2, "type": "ssd",
         "name": "ubuntu-1804-bionic",
         "_id": "5bef002c680cffd11f107590"}
    ],
    "netadp": [
        {
            "ip": [{"address": "62.181.8.174"}],
            "network": {
                "dns": {"nameservers": ["8.8.8.8", "8.8.4.4"]},
                "routing": [],
                "gateway": "62.181.8.1",
                "netmask": "255.255.248.0",
                "name": "public",
                "type": "public",
                "_id": "5784e97be2627505227b578c"
            },
            "speed": 1000,
            "type": "hv",
            "macaddress": "00:15:5D:FF:0F:03",
            "_id": "5beab450680cffd11f0e6102"
        },
        {
            "ip": [{"address": "10.209.78.11"}],
            "network": {
                "dns": {"nameservers": ["9.9.9.9", "8.8.8.8"]},
                "routing": [],
                "gateway": "10.209.78.1",
                "netmask": "255.255.255.0",
                "name": "network-determined-bardeen",
                "type": "private",
                "_id": "5beaec64680cffd11f0e7c31"
            },
            "speed": 1000,
            "type": "hv",
            "macaddress": "00:15:5D:FF:0F:24",
            "_id": "5bec18c6680cffd11f0f0d8b"
        }
    ],
    "dvddrive": [{"iso": {}}]
}


class TestRbxDataSource(CiTestCase):
    parsed_user = None
    allowed_subp = ['bash']

    def _fetch_distro(self, kind):
        cls = distros.fetch(kind)
        paths = helpers.Paths({})
        return cls(kind, {}, paths)

    def setUp(self):
        super(TestRbxDataSource, self).setUp()
        self.tmp = self.tmp_dir()
        self.paths = helpers.Paths(
            {'cloud_dir': self.tmp, 'run_dir': self.tmp}
        )

        # defaults for few tests
        self.ds = ds.DataSourceRbxCloud
        self.seed_dir = self.paths.seed_dir
        self.sys_cfg = {'datasource': {'RbxCloud': {'dsmode': 'local'}}}

    def test_seed_read_user_data_callback_empty_file(self):
        populate_user_metadata(self.seed_dir, '')
        populate_cloud_metadata(self.seed_dir, {})
        results = ds.read_user_data_callback(self.seed_dir)

        self.assertIsNone(results)

    def test_seed_read_user_data_callback_valid_disk(self):
        populate_user_metadata(self.seed_dir, '')
        populate_cloud_metadata(self.seed_dir, CLOUD_METADATA)
        results = ds.read_user_data_callback(self.seed_dir)

        self.assertNotEqual(results, None)
        self.assertTrue('userdata' in results)
        self.assertTrue('metadata' in results)
        self.assertTrue('cfg' in results)

    def test_seed_read_user_data_callback_userdata(self):
        userdata = "#!/bin/sh\nexit 1"
        populate_user_metadata(self.seed_dir, userdata)
        populate_cloud_metadata(self.seed_dir, CLOUD_METADATA)

        results = ds.read_user_data_callback(self.seed_dir)

        self.assertNotEqual(results, None)
        self.assertTrue('userdata' in results)
        self.assertEqual(results['userdata'], userdata)

    def test_generate_network_config(self):
        expected = {
            'version': 1,
            'config': [
                {
                    'subnets': [
                        {'control': 'auto',
                         'dns_nameservers': ['8.8.8.8', '8.8.4.4'],
                         'netmask': '255.255.248.0',
                         'address': '62.181.8.174',
                         'type': 'static', 'gateway': '62.181.8.1'}
                    ],
                    'type': 'physical',
                    'name': 'eth0',
                    'mac_address': '00:15:5d:ff:0f:03'
                },
                {
                    'subnets': [
                        {'control': 'auto',
                         'dns_nameservers': ['9.9.9.9', '8.8.8.8'],
                         'netmask': '255.255.255.0',
                         'address': '10.209.78.11',
                         'type': 'static',
                         'gateway': '10.209.78.1'}
                    ],
                    'type': 'physical',
                    'name': 'eth1',
                    'mac_address': '00:15:5d:ff:0f:24'
                }
            ]
        }
        self.assertTrue(
            ds.generate_network_config(CLOUD_METADATA['netadp']),
            expected
        )

    @mock.patch(DS_PATH + '.subp.subp')
    def test_gratuitous_arp_run_standard_arping(self, m_subp):
        """Test handle run arping & parameters."""
        items = [
            {
                'destination': '172.17.0.2',
                'source': '172.16.6.104'
            },
            {
                'destination': '172.17.0.2',
                'source': '172.16.6.104',
            },
        ]
        ds.gratuitous_arp(items, self._fetch_distro('ubuntu'))
        self.assertEqual([
            mock.call([
                'arping', '-c', '2', '-S',
                '172.16.6.104', '172.17.0.2'
            ]),
            mock.call([
                'arping', '-c', '2', '-S',
                '172.16.6.104', '172.17.0.2'
            ])
        ], m_subp.call_args_list
        )

    @mock.patch(DS_PATH + '.subp.subp')
    def test_handle_rhel_like_arping(self, m_subp):
        """Test handle on RHEL-like distros."""
        items = [
            {
                'source': '172.16.6.104',
                'destination': '172.17.0.2',
            }
        ]
        ds.gratuitous_arp(items, self._fetch_distro('fedora'))
        self.assertEqual([
            mock.call(
                ['arping', '-c', '2', '-s', '172.16.6.104', '172.17.0.2']
            )],
            m_subp.call_args_list
        )

    @mock.patch(
        DS_PATH + '.subp.subp',
        side_effect=subp.ProcessExecutionError()
    )
    def test_continue_on_arping_error(self, m_subp):
        """Continue when command error"""
        items = [
            {
                'destination': '172.17.0.2',
                'source': '172.16.6.104'
            },
            {
                'destination': '172.17.0.2',
                'source': '172.16.6.104',
            },
        ]
        ds.gratuitous_arp(items, self._fetch_distro('ubuntu'))
        self.assertEqual([
            mock.call([
                'arping', '-c', '2', '-S',
                '172.16.6.104', '172.17.0.2'
            ]),
            mock.call([
                'arping', '-c', '2', '-S',
                '172.16.6.104', '172.17.0.2'
            ])
        ], m_subp.call_args_list
        )


def populate_cloud_metadata(path, data):
    populate_dir(path, {'cloud.json': json.dumps(data)})


def populate_user_metadata(path, data):
    populate_dir(path, {'user.data': data})
